// pmMeshPaint Plugin

#pragma warning (disable:4800)
#pragma warning (disable:4305)	

#define GLEW_STATIC
#include "includes.h"

#define MOUSE_MOVE_TIME_DELAY 3

SICALLBACK XSILoadPlugin( PluginRegistrar& in_reg )
{

	in_reg.PutAuthor(L"piotrek marczak");
	in_reg.PutName(L"pmMeshPaint Plugin");
	in_reg.PutVersion(0,95);
	in_reg.RegisterTool(L"pmMeshPaint");	

	//RegistrationInsertionPoint - do not remove this line
	
	return CStatus::OK;
}

SICALLBACK XSIUnloadPlugin( const PluginRegistrar& in_reg )
{

	return CStatus::OK;
}

////////////////////////////////////////////////////////////////////////////////
//
// pmQuickPrimitive
//
////////////////////////////////////////////////////////////////////////////////

class pmMeshPaint {
private: // Data
	CString	m_pluginPath;
	// ---------------------- mouse data
	LONG m_mouseDownX,m_mouseDownY;
	LONG m_mouseDragX,m_mouseDragY;
	LONG m_mouseLastDragX, m_mouseLastDragY;
	LONG m_mouseBezierX,m_mouseBezierY;
	LONG m_mouseX,m_mouseY;
	POINT lp_mouse; //absolute mouse position
	CStatus m_lastMouseMoveStatus;
	// ----------------------

	enum enToolMode {kMenu,kDrawing,kDrawingDrop,kBrushRadius,kButtonDrag,kIdle} m_toolMode;
//	enum enPaintMode {kBrushPaint,kDrop} ;
	enum  enAxis{kx=0,ky,kz} ;
	// ---------------- picked data
	enum enSubcomponentType{kVertex,kEdge, kPoly, kSurface, kNone}  m_pickedSubcompType;

	X3DObject	m_picked;

	Edge		m_pickedEdge;
	PolygonFace m_pickedPolyFace;
	Vertex		m_pickedVertex;
	CVector3	m_pickedSubcompPos, //position of subcomponent
				m_pickedNormal,	//normal of subcomponent
				m_pickedPosition;
	CVector3	m_pickedFrameAxisX,m_pickedFrameAxisY,m_pickedFrameAxisZ;
				
	LONG		m_activeAxis;		// pick axis during mouse move
	
	bool		m_lineDrawing;		// stroke is drawn in line
	bool		m_bezierLineDrawing;	//bezier change when line drawing in progress
	// ----------------------

	clock_t		m_tooltime,
				m_dragtime;
	CVector3	m_draglastpickedPos;	//for distance based cloning
	//preferences
	double		m_snapIncrement;
	int			m_planeScale;
	int			m_axisScale;

	// cache for options from PPG (read once from PPG in mouse down event, read many times from this struct in mouse drag)
	struct {
		bool	drawcomponenthighlight;
		
		bool	paintonselection;
		bool	DeleteOverlaps;
		double	BBoxScale;
		enAxis	alignaxis;
		LONG	gridpaint;
		double  buildtime;
		LONG    updatemode;
		
		double  builddist;
		LONG	stackclones,randsclrange, randscluni, 	scaleramp, scalerampuni;
		double  randposx,randposy,randposz,randrotx,randroty,randrotz,randsclx,randscly,randsclz, 
				scx,scy,scz,	axisoffsetx,axisoffsety,axisoffsetz,	rotoffsetx,rotoffsety,rotoffsetz,	scalex,scaley,scalez,
				scaleminx,scaleminy,scaleminz;
		LONG	randseed, paintinstances, cloneCount,paintgrid, Align, randlocalTrans, listpick, AlignAxisFlip, AlignAxis, FollowStroke, localTrans;

		CString	groupName, nullName;
		double	StrokeSmoothLength;
		
	} m_options;
	
	CustomProperty		ppg;				// property retrieved from global data	

	BrushTool			m_brush;			//brush
	CRefArray			m_paintClones;		//objects to clone from PPG

	LONG				m_cloneId;
	// clone data
	struct	s_stroke {
		CRefArray			strokeclones;			// ref list of last stroke clones
		CVector3			strokeCenter;			// stroke center (mouse position /raycasted/)
		LONG				dupCount;
		vector<CVector3>	rotx;					// store initial rotation
		vector<CVector3>	roty;
		vector<CVector3>	rotz;
		vector<CRotation>	rotrand;				// store random rotation
		vector<CVector3>	sclrand;				// store random scale
		vector<CVector3>	transrand;				// store random translation
		vector<CVector3>	posOffset;				// offset of position from center of stroke
		vector<CTransformation>		transform;				// store transformation...
		vector<bool>		flag;					// general purpose flag (now used in keep apart)...
		CRef				strokeSourceObj;
	};

	vector<s_stroke>	m_strokes;				//all strokes
	vector<LONG>		m_strokeQueueIndex;		//for later duplication
	// misc data
	bool				m_screenspacebrush;		// always true
	PickBuffer			m_pickbuffer, m_pickbufferObj;	// pickbuffer caches

	//	holds group & null objects for clone grouping
	Group				m_cloneGroup;		
	X3DObject			m_cloneNullParent;
	//
	

	// *** DEBUG helper variables
	CVector3			dumpvector[4];
	LONG				dumppixels[4];
	vector<double>		dumpv;
	LONG				cnta, cntb;
	vector<CVector3> debugVE,debugVS;
	//****

public: // Methods
	pmMeshPaint()
	{}

	~pmMeshPaint()
	{}
	// read source objects from PPG
	CStatus GetPaintObjectsFromPPG(ToolContext& in_ctxt, CRefArray out_list)
	{
		Model oRoot = Application().GetActiveSceneRoot();
		CStringArray sz_objstrarray;
		
		if (!ppg.IsValid() || ppg.GetFullName() == L"")
			return CStatus::Fail;

		if ( (LONG)ppg.GetParameter(L"paintselected").GetValue() == 0 )
		{
			PPGLayout ppgLayout = ppg.GetPPGLayout();
			PPGItem item = ppgLayout.GetItem(L"objList");
			CValueArray items = item.GetUIItems();
			for (int i=0;i<items.GetCount();i+=2)	
				sz_objstrarray.Add(items[i].GetAsText());	
		}
		else	{
			Parameter objList = ppg.GetParameter(L"objList");
			CString szobjlist = objList.GetValue();	
			
			sz_objstrarray = szobjlist.Split(L";");
		}
		
		CStringArray strFamilies;
		strFamilies.Add( si3DObjectFamily );
		
		out_list.Clear();
		
		for (int i=0;i < sz_objstrarray.GetCount();i++)
		{

			X3DObject fobj = oRoot.FindChild2(sz_objstrarray[i],L"",strFamilies);
			if (fobj.IsValid())	{
				out_list.Add(fobj);
			}
			else				{				
				CRefArray groupsArray = oRoot.GetGroups();
				for ( LONG j=0; j < groupsArray.GetCount(); j++ )
				{
					Group lgroup = groupsArray[j];
					if (lgroup.GetName() == sz_objstrarray[i] )	
					{						
						out_list += lgroup.GetMembers();	
						break;
					}
				}	//groups lookup				
			}
		}	// split str loop
		return CStatus::OK;
	}
	CStatus GetGroup( CString l_grpname, Group &out_group )
	{
		Model oRoot = Application().GetActiveSceneRoot();
		CRefArray groupsArray = oRoot.GetGroups();
		for ( LONG j=0; j < groupsArray.GetCount(); j++ )
		{
			Group lgroup = groupsArray[j];
			if (lgroup.GetName() == l_grpname )	{
				out_group = lgroup;	
				return CStatus::OK;
			}
		}	//groups lookup
		return CStatus::Fail;
	}

	CStatus GetNull (CString l_nullname, X3DObject &out_null)
	{
		Model oRoot = Application().GetActiveSceneRoot();
		CStringArray families;
		families.Add(siNullPrimitiveFamily);
		out_null = oRoot.FindChild2(l_nullname, siNullPrimType,families);
		if ( out_null.IsValid() )
			return CStatus::OK;
		return CStatus::Fail;
	}

	CStatus Setup( ToolContext& in_ctxt )
	{
		Application().LogMessage( L"pmMeshPaint" );
		Plugin thisPlugin = Application().GetPlugins().GetItem( L"pmMeshPaint Plugin" ) ;
		
		m_pluginPath = thisPlugin.GetOriginPath() ;

		ClearAllData();

		if( GLEW_OK != glewInit() ){
			logmsg("Couldn't initialize GLEW");
			return CStatus::Abort;
		}
		// Set default values
		m_axisScale = 30;
		m_brush.Setup();
		m_toolMode = kIdle;
		
		m_options.FollowStroke = 0;
		m_options.alignaxis= ky;
		m_options.updatemode = 0;
		
		m_options.drawcomponenthighlight = true;
		m_screenspacebrush = true;
		m_options.stackclones = 0;
		m_cloneId = 0;
		
		// Get link to PPG
		CRef ppgref;
		CStatus ppgStatus =  GetSavedData(L"pmMeshPaintPPG_GlobalPPG_Link", ppg);
		if ( ppgStatus != CStatus::OK || !ppg.IsValid() ) 	{
			logmsg("Couldn't find meshPaint PPG");
			return CStatus::Abort;
		}
		
		// Get saved data
		double xyzsaved;
		if ( GetSavedData(L"pmMeshPaint_BrushSize",xyzsaved) == CStatus:: OK)	{
			m_brush.SetRadius(xyzsaved);
			m_brush.SetBaseRadius(xyzsaved);
		}
		if ( GetPaintObjectsFromPPG( in_ctxt, m_paintClones ) != CStatus::OK )	{

		}
		if (! m_paintClones.GetCount() )	{
			logmsg("No objects in object list!");
		}
		in_ctxt.EnableSnapping( true );
		Activate( in_ctxt );
		cnta=cntb=0;
		return CStatus::OK;
	}

	CStatus Cleanup( ToolContext& in_ctxt )
	{
		in_ctxt.SetCursor( siArrowCursor );

		SaveData((double)m_brush.GetRadius(),L"pmMeshPaint_BrushSize");

		ClearAllData();

		LogOpenGLErrors();

		return CStatus::OK;
	}

	CStatus Activate( ToolContext& in_ctxt )
	{
		// Called when tool becomes the active tool
		in_ctxt.SetCursor( siCrossCursor );

		m_tooltime = clock();

		in_ctxt.RegisterShortcutKey( kKeyR, L"Brush Radius" );
		
		in_ctxt.SetToolDescription( L"pmMeshPaint\nPaint\nDuplicate Last\nMenu" );
		
#ifdef _DEBUG
		logmsg(L"TOOL ACTIVE");
#endif
		return CStatus::OK;
	}

	CStatus Deactivate( ToolContext& in_ctxt )
	{
		in_ctxt.SetCursor( siArrowCursor );
		// Called when tool becomes inactive (e.g. when user orbits using nav or chooses another tool)
		return CStatus::OK;
	}
	CStatus readOptionsFromPPG()
	{
			if (ppg.GetFullName() == L"" || !ppg.IsValid() )
				return CStatus::Fail;
			m_options.gridpaint =  (LONG)ppg.GetParameter(L"paintgrid").GetValue();
			//m_options.buildtime  = ppg.GetParameter(L"builduptime") .GetValue();
		//m_options.updatemode = (LONG) ppg.GetParameter(L"updatemode").GetValue();
			m_options.builddist  = (double)ppg.GetParameter(L"buildupdist").GetValue();
			m_options.stackclones = (LONG)ppg.GetParameter(L"stackclones").GetValue();
			m_options.randposx =  (double)ppg.GetParameter(L"randposx").GetValue();
			m_options.randposy =  (double)ppg.GetParameter(L"randposy").GetValue();
			m_options.randposz =  (double)ppg.GetParameter(L"randposz").GetValue();
			m_options.randrotx =  (double)ppg.GetParameter(L"randrotx").GetValue();
			m_options.randroty =  (double)ppg.GetParameter(L"randroty").GetValue();
			m_options.randrotz =  (double)ppg.GetParameter(L"randrotz").GetValue();
			m_options.randsclx =  (double)ppg.GetParameter(L"randsclx").GetValue();
			m_options.randscly =  (double)ppg.GetParameter(L"randscly").GetValue();
			m_options.randsclz =  (double)ppg.GetParameter(L"randsclz").GetValue();
			m_options.scx = (double)ppg.GetParameter(L"scalex").GetValue();
			m_options.scy = (double)ppg.GetParameter(L"scaley").GetValue();
			m_options.scz = (double)ppg.GetParameter(L"scalez").GetValue();
			m_options.scaleramp = (LONG) ppg.GetParameter(L"scaleramp").GetValue();
		
			m_options.scalerampuni = (LONG)ppg.GetParameter(L"scalerampuni").GetValue();
			m_options.scalex = (double)ppg.GetParameter(L"scalex").GetValue();
			m_options.scaley = (double)ppg.GetParameter(L"scaley").GetValue();
			m_options.scalez = (double)ppg.GetParameter(L"scalez").GetValue();
			m_options.scaleminx = (double)ppg.GetParameter(L"scaleminx").GetValue();
			m_options.scaleminy = (double)ppg.GetParameter(L"scaleminy").GetValue();
			m_options.scaleminz = (double)ppg.GetParameter(L"scaleminz").GetValue();
			m_options.rotoffsetx = (double)ppg.GetParameter(L"rotoffsetx").GetValue();
			m_options.rotoffsety = (double)ppg.GetParameter(L"rotoffsety").GetValue();
			m_options.rotoffsetz = (double)ppg.GetParameter(L"rotoffsetz").GetValue();
			m_options.axisoffsetx = (double)ppg.GetParameter(L"axisoffsetx").GetValue();
			m_options.axisoffsety = (double)ppg.GetParameter(L"axisoffsety").GetValue();
			m_options.axisoffsetz = (double)ppg.GetParameter(L"axisoffsetz").GetValue();
			m_options.randsclrange = (LONG)ppg.GetParameter(L"randsclrange").GetValue();
			m_options.randscluni = (LONG)ppg.GetParameter(L"randscluni").GetValue();
			m_options.randseed = (LONG)ppg.GetParameter(L"randseed").GetValue();
			m_options.paintinstances = (LONG)ppg.GetParameter(L"paintinstances").GetValue();
			m_options.cloneCount = (LONG)ppg.GetParameter(L"CloneCount").GetValue();
			m_options.paintgrid = (LONG)ppg.GetParameter(L"paintgrid").GetValue();
			m_options.Align = (LONG)ppg.GetParameter(L"Align_To_Surface").GetValue();
			m_options.FollowStroke = (LONG)ppg.GetParameter(L"FollowStroke").GetValue();
			m_options.AlignAxis = (LONG)ppg.GetParameter(L"AlignAxis").GetValue();
			m_options.randlocalTrans = (LONG)ppg.GetParameter(L"randlocalTrans").GetValue();
			m_options.localTrans = (LONG)ppg.GetParameter(L"localTrans").GetValue();
			m_options.listpick = (LONG)ppg.GetParameter(L"listpick").GetValue();
			m_options.AlignAxisFlip = (LONG)ppg.GetParameter(L"AlignAxisFlip").GetValue();

			m_options.groupName = (CString) ppg.GetParameter(L"groupName").GetValue();
			m_options.nullName = (CString) ppg.GetParameter(L"nullName").GetValue();
			m_options.StrokeSmoothLength = (double)ppg.GetParameter(L"StrokeSmoothLength").GetValue();
			
			m_options.DeleteOverlaps = (LONG)ppg.GetParameter(L"DeleteOverlaps").GetValue();
			m_options.BBoxScale = (double)ppg.GetParameter(L"BBoxScale").GetValue();
			switch (m_options.AlignAxis){
				case 0: m_options.alignaxis = kx;break;
				case 1: m_options.alignaxis = ky;break;
				case 2: m_options.alignaxis = kz;break;
				default: m_options.alignaxis = ky;break;
			}
			/*
			switch ( (LONG)ppg.GetParameter(L"Paint_Mode").GetValue() )	{
				case 1: m_options.paintMode = kDrop;break;
				case 0:
				default: m_options.paintMode = kBrushPaint;break;
			}*/
			return CStatus::OK;
	};
	// **********************************************************************************************
	//														MOUSE DOWN
	// **************************************************************************************************

	CStatus MouseDown( ToolContext& in_ctxt )
	{
		// Called when mouse button is pressed
		LONG x, y;
		in_ctxt.GetMousePosition( x, y );
		m_mouseBezierX = m_mouseDownX = m_mouseLastDragX = m_mouseDragX = x;
		m_mouseBezierY = m_mouseDownY = m_mouseDragY = m_mouseLastDragY =y;
		
		m_toolMode = kIdle;
		m_bezierLineDrawing = false;
		
		if ( in_ctxt.IsLeftButtonDown() || in_ctxt.IsRightButtonDown() )	
		{		
			if ( in_ctxt.IsShortcutKeyDown(kKeyR) )	{
				if ( (LONG)ppg.GetParameter(L"Paint_Mode").GetValue() == 0 )	//Brush mode
				{
					m_toolMode = kBrushRadius;
					ShowCursor(NULL);
					GetCursorPos(&lp_mouse);
					return CStatus::OK;
				}
				else
					return CStatus::False;
			}

			// Cache parameters from PPG (object list and options)
			GetPaintObjectsFromPPG(in_ctxt, m_paintClones);
			readOptionsFromPPG();
			if ( !m_paintClones.GetCount() )	{	
				//no object(s) to paint
				LONG paintWithsel = (LONG)ppg.GetParameter(L"paintwithselection").GetValue();
				if (paintWithsel == 0)
					return CStatus::False;
				// *******************
				// Paint with selection - get selection members
				// *******************
				Selection sel = Application().GetSelection();
				if ( !sel.GetCount() )
					return CStatus::False;
				CRefArray objlist = sel.GetArray();
				
				for (int i=0;i < objlist.GetCount();i++)
				{				
					switch ( objlist[ i ].GetClassID() ){
						case siGroupID:
							{
								Group g = objlist[ i ];
								m_paintClones += g.GetMembers();
							}
							break;
						case siX3DObjectID:
							m_paintClones.Add( objlist[i] );
							break;
						case siCameraRigID:
						case siLightRigID:            
						case siModelID:  
						case siCameraID:     
						case siLightID:
						case siNullID:
							m_paintClones.Add( objlist[i] );
							break;					
						default:
							break;			
					}
				}	

				if (!m_paintClones.GetCount() )
					return CStatus::False;
			}
			//	get selection under cursor
			CVector3 l_pos;
			
			m_strokes.clear();

			bool rmb = in_ctxt.IsRightButtonDown();
			
			if( (LONG)ppg.GetParameter(L"Paint_Mode").GetValue() == 0 )
				m_toolMode = rmb ? kDrawingDrop : kDrawing;
			else
				m_toolMode = rmb ? kDrawing : kDrawingDrop;
			
			m_lineDrawing = in_ctxt.IsShiftKeyDown() && m_toolMode ==kDrawing;
			m_dragtime = 0;
			m_pickedPosition = m_draglastpickedPos = l_pos;

			clock_t timer = clock();
			CRefArray in_objects;
			CRefArray in_compobjects;
			LONG paintOnsel = (LONG)ppg.GetParameter(L"paint_on_selection_only").GetValue();
			if ( paintOnsel != 0)	
				in_objects = Application().GetSelection().GetArray();
			
			m_pickbufferObj = in_ctxt.GetPickBuffer(0,0,0,0,siObjectFilter, in_objects );

			if ( paintOnsel != 0)	
				in_compobjects = in_objects;
			else	{
				CStringArray strFamilies;
				strFamilies.Add( siMeshFamily );
	
				Model oRoot = Application().GetActiveSceneRoot();
				in_compobjects = oRoot.FindChildren(L"",L"",strFamilies);
			}

			m_pickbuffer = in_ctxt.GetPickBuffer(0,0,0,0,siPolygonFilter, in_compobjects );

			// if group doesn't exists in scene but string prop is not empty, create group. Same with null
			if ( GetGroup( m_options.groupName, m_cloneGroup) != CStatus::OK && m_options.groupName != L"" )	{
				m_cloneGroup = Group();
				Application().GetActiveSceneRoot().AddGroup( CRefArray(), m_options.groupName, true, m_cloneGroup );
			}
			if ( GetNull( m_options.nullName, m_cloneNullParent) != CStatus::OK && m_options.nullName != L"" )	{
				m_cloneNullParent = X3DObject();
				Application().GetActiveSceneRoot().AddPrimitive( L"null", m_options.nullName, m_cloneNullParent );
			}
			in_ctxt.GetWorldPosition(x,y,l_pos);
			m_draglastpickedPos = l_pos;
			//fist click, without dragging, should also create clone(s)
			bool gridpicked = true;
			if ( m_pickbufferObj.GetObjectAtPosition(x,y).IsValid() )
				gridpicked = false;
			if ( UpdateMouseDrag(in_ctxt, x, y, gridpicked, true, false, CVector3(0,0,0) ) != CStatus::OK)	
				if (m_toolMode == kDrawingDrop && !m_strokes.size() )
				{	
					m_toolMode = kIdle;
					m_activeAxis = kNoDragAxis;
					in_ctxt.Redraw( false );
					return CStatus::False;
				}
			
			in_ctxt.SetCursor(siDotCursor);

			return CStatus::OK;
		}
		else
				if ( in_ctxt.IsMiddleButtonDown() )		
				{
					return CStatus::False; //ignore mouse up callback
				}
		return CStatus::False; // Ignore other mouse buttons etc.
	}
	
	CStatus gridOrObjectUnderMouseCursor(ToolContext& in_ctxt , LONG x, LONG y, CVector3 &l_pos, bool &l_gridpicking)
	{
		CRef objAtmouse = m_pickbufferObj.GetObjectAtPosition( x, y );
		l_gridpicking = !objAtmouse.IsValid();
		if ( l_gridpicking )
		{
			CStatus pickStatus;
			
			if(m_options.paintgrid)	{
				CPlane gridPlane;
				pickStatus = GetGridData(in_ctxt,x,y,l_pos,gridPlane);
				// Reject intersection points behind the camera
				if (pickStatus == CStatus::OK)	{
					CVector3 cameraPoint;
					if ( in_ctxt.WorldToCamera( l_pos, cameraPoint ) != CStatus::OK || cameraPoint.GetZ() > 0.0 )
						l_gridpicking = false;
				}
				else
					l_gridpicking = false;

			}
			else
				l_gridpicking = false;

			if (!l_gridpicking)	{
				ClearPickedData();
				return CStatus::False;
			}
		}
		else
			in_ctxt.GetWorldPosition(x,y,l_pos);
		return CStatus::OK;
	}
	
	// **********************************************************************************************
	//														MOUSE DRAG
	// **************************************************************************************************

	CStatus MouseDrag( ToolContext& in_ctxt )
	{
		LONG x, y;
		in_ctxt.GetMousePosition( x, y );
		
		m_mouseX = x;
		m_mouseY = y;
		bool bezierchange = false;
		if ( m_lineDrawing )	{
			if( in_ctxt.IsRightButtonDown() )	{
				bezierchange = true;
				m_bezierLineDrawing = true;
			}
			else
				m_bezierLineDrawing = false;
		}
		if ( in_ctxt.IsLeftButtonDown() ||  in_ctxt.IsRightButtonDown()  )
		{
			// change brush radius
			if (m_toolMode == kBrushRadius)					
			{			
				CVector3 md,mdown;
				
				in_ctxt.GetWorldPosition(x,y, md);
				in_ctxt.GetWorldPosition(m_mouseDownX,m_mouseDownY, mdown);	
				md.SubInPlace(mdown);
			
				double len = md.GetLength();

				if (x- m_mouseDownX < 0.0 ) len*=-1.0;
				m_brush.SetRadius( m_brush.GetBaseRadius() +  len );
				return CStatus::OK;
			}
			// object painting
			if (m_toolMode == kDrawing || m_toolMode == kDrawingDrop)
			{			
				clock_t dragTimer = clock();
								
				// ***************** painting main *****************
				
				if (m_lineDrawing)	{
					in_ctxt.EnableSnapping( false );
					LONG ex=x, ey=y;
					if ( bezierchange )	{
						ex = m_mouseDragX;
						ey = m_mouseDragY;
						m_mouseBezierX = x;
						m_mouseBezierY = y;
					}
					else	
						if( m_bezierLineDrawing ){
							m_mouseBezierX = ex;
							m_mouseBezierY = ey;
						}
					UpdateMouseDragLine(in_ctxt, m_mouseDownX, m_mouseDownY, ex, ey, false);
					in_ctxt.EnableSnapping( true );
				}
				else
				if (m_toolMode != kDrawingDrop)	
				{
					CVector3 l_pos;

					// check if there is an object or grid under cursor, otherwise return
					bool l_gridpicking;
					if ( gridOrObjectUnderMouseCursor( in_ctxt, x, y , l_pos, l_gridpicking) != CStatus::OK)
						return CStatus::OK;	//continue
	
					m_pickedSubcompPos = l_pos;
					in_ctxt.GetWorldPosition(x,y,l_pos);
					CVector3 distance = l_pos;
					distance -= m_draglastpickedPos;

					if(  distance.GetLength() >= m_options.builddist )
					{	
						if ( (LONG)ppg.GetParameter(L"interpolateposition").GetValue() != 0)					
						{
							in_ctxt.EnableSnapping( false );
							UpdateMouseDragLine(in_ctxt, m_mouseLastDragX, m_mouseLastDragY, x, y, true);
							in_ctxt.EnableSnapping( true );
						}
						else
							UpdateMouseDrag(in_ctxt, x, y, l_gridpicking, true,false, CVector3(0,0,0));
						m_dragtime = 0;
						
						m_draglastpickedPos = l_pos;
						m_mouseLastDragX = x;
						m_mouseLastDragY = y;
					//	return CStatus::OK;				
					}
				}
				else	// ***************** drop method	*****************														
				{
					CVector3	len,mouseDragV, mousedownvect ;
					CVector3	upv = m_pickedFrameAxisY;
					CPlane		cp( m_pickedPosition, upv );
					
					in_ctxt.SetManipulationPlane(cp);
					in_ctxt.GetWorldPosition(x,y, mouseDragV);
					in_ctxt.GetWorldPosition(m_mouseDownX,m_mouseDownY, mousedownvect);		
					in_ctxt.ClearManipulationPlane();
					double u,v;
					cp.GetParameters( mouseDragV, u, v);
					mouseDragV = cp.GetPosition( u,v);
					cp.GetParameters( mousedownvect, u, v);
					mousedownvect = cp.GetPosition( u,v);
					
					len.Sub(mouseDragV, mousedownvect);
					double lend = len.GetLength();
					CVector3 bboxmin, bboxmax, bboxhalf;
					X3DObject srcobj = m_strokes[0].strokeSourceObj;
						
					if ( srcobj.GetType() == L"polymsh" ||  srcobj.GetType() == L"#model" )
					{
						CTransformation ct;
						ct.SetScalingFromValues(1,1,1);
						if (srcobj.GetType() == L"#model" )
							GetModelBoudingBox( srcobj, bboxmin, bboxmax, bboxhalf, true, ct);
						else
							GetBoudingBox( srcobj, bboxmin, bboxmax,bboxhalf, true, ct);
						bboxmin -= bboxmax;
						lend /= (bboxmin.GetLength()*0.5);
					}
					
					for(LONG j=0;j < m_strokes.size(); j++)
						for(LONG i=0;i<m_strokes[j].strokeclones.GetCount();i++)
						{
							X3DObject l_obj = m_strokes[j].strokeclones[i];
							KinematicState kts = l_obj.GetKinematics().GetGlobal();
							CTransformation kt ( kts.GetTransform() );
							
							kt.SetScalingFromValues( lend, lend, lend );	//set scale
							// calc rotation
							CVector3 ax, ay, az;
							ax = len;
							
							ax.NormalizeInPlace();
							ay = upv;
							
							// ****************** rotate SNAP ************************
							if ( in_ctxt.IsShiftKeyDown() )
							{
								CVector3 forw_ref,up_ref;
								forw_ref = m_pickedFrameAxisX;
								up_ref = m_pickedFrameAxisY;

								len.NormalizeInPlace();
								double ang = forw_ref.GetAngle( len );	// angle between direction of mouse vector and input forward vect
								//determine sign of rotation (+-180)
								CVector3 cross= cross.Cross( m_pickedFrameAxisY, m_pickedFrameAxisX );
								double sign = len.Dot( cross ) > 0.0f ? 1.0 : -1.0;
								// get rotation snapping angle from preferences
								CValue ret;
								Application().GetPreferences().GetPreferenceValue(L"TransformProperties.rotatesnap",ret);
								double newAngle = DegreesToRadians ( GetClosestIncrement(  RadiansToDegrees( ang ) , (double)ret ) )*sign;
								// construct rotation matrix
								double cosa = cos( newAngle );
								double sina = sin( newAngle );
								double t = 1.0 - cosa;							
								
								CVector3 vn = m_pickedFrameAxisY;							
								double x=vn.GetX();	double y=vn.GetY();	double z=vn.GetZ();
								CMatrix3 m2;
								m2.Set(	t*x*x+cosa,t*x*y+sina*z,t*x*z-sina*y,
										t*x*y - sina*z, t*y*y+cosa, t*y*z +sina*x,
										t*x*z + sina*y, t*y*z - sina*x, t*z*z + cosa
										);
								// rotate vector by multiplying forward vector by rotaion matrix
								CVector3 out_v=forw_ref;
								out_v*=m2;
								out_v.NormalizeInPlace();
								ax = out_v;
							}
							// ************** Finalize rotation
							az.Cross(ax,ay);
							CVector3 temp_ry;
							switch (m_options.alignaxis)	{	
								case ky:
									if (m_options.AlignAxisFlip )
										ay.NegateInPlace();
									az.Cross(ax,ay);
									break;
								case kx:
									if (m_options.AlignAxisFlip)
										ax.NegateInPlace();
									temp_ry = ax;
									ax = ay;
									ay = temp_ry;
									az.Cross(ax,ay);
									break;
								case kz:
									if (m_options.AlignAxisFlip)
										az.NegateInPlace();
									temp_ry = az;
									az = ay;
									ay = temp_ry;
									ax.Cross(ay, az);
									break;
							}
							kt.SetRotationFromXYZAxes(ax,ay,az);
							in_ctxt.UpdateKinematicState( kts, kt);
						}
				}

				if ( (m_lineDrawing && !bezierchange) || !m_lineDrawing )	{
					m_mouseDragX = x;
					m_mouseDragY = y;
				}
				m_dragtime += ( clock() - dragTimer );
				

			}
		}

		return CStatus::OK;
	}	
	
	CQuaternion ApplyRotation(const CRotation &in_rand_rot, const CRotation &in_rot)
	{
		CRotation offsetrot, randomRot;
		offsetrot.SetFromXYZAngles( DegreesToRadians(m_options.rotoffsetx ),DegreesToRadians(m_options.rotoffsety ),DegreesToRadians(m_options.rotoffsetz ));
		
		CQuaternion quat = offsetrot.GetQuaternion();
		quat.MulInPlace(in_rot.GetQuaternion());
		return quat.MulInPlace(in_rand_rot.GetQuaternion());
	}

	void Duplicate(CRef in_obj, LONG l_dupcount, bool paintinstances, CRefArray &out_clones)
	{
	
		CValue outArg;
		CValueArray a(19); 
				
		CString duplicateFunc;
		/*
			If preferences paint instances is false
		*/
		CString l_objClass = in_obj.GetClassIDName();
		X3DObject clonethis = in_obj;
		
		if ( l_objClass == L"Model" )	{
			if (paintinstances)
				duplicateFunc = L"Instantiate";
			else
				duplicateFunc = L"Clone";
		}
		else
			if (l_objClass == L"X3DObject")
			{
				X3DObject xobj( in_obj );

				if ( paintinstances && xobj.GetType() == L"polymsh")
					duplicateFunc = L"Clone";
				else
					duplicateFunc = L"SIDuplicate";
			}
			else
				duplicateFunc = L"SIDuplicate";
						
		if (duplicateFunc == L"Clone")	{
			if ( l_objClass == L"Model" )
				a[0] = CValue( L"B:"+ clonethis.GetFullName() );
			else
				a[0] = CValue( clonethis.GetFullName() );
			a[1] = CValue( LONG(l_dupcount) );

			a[7] = CValue( L"siNoSelection");
		}
		else 
			if (duplicateFunc == L"Instantiate")	{
				a.Resize(16);
				a[0] = CValue( clonethis.GetFullName() );
				a[1] = CValue( LONG(l_dupcount) );
				a[4] = CValue( L"siNoSelection");
			}
			else
				if (duplicateFunc == L"SIDuplicate")	{
					a.Resize(10);
					if ( l_objClass == L"Model" )
						a[0] = CValue( L"B:"+ clonethis.GetFullName() );
					else
						a[0] = CValue( clonethis.GetFullName() );
						
					a[1] = CValue( LONG(l_dupcount) );
				}
#ifdef _DEBUG
		logmsg(L"dup func", duplicateFunc);
		CString lstr;
		for (int i=0;i < a.GetCount();i++)
			lstr += a[i].GetAsText()+L",";
		logmsg(L"args ", lstr);
#endif
		Application().ExecuteCommand(duplicateFunc, a, outArg);
		out_clones = outArg;
			
		/*
		else	//clone only geometry (not using command)
			{
				if (m_paintClones[cloneind].GetClassIDName() == L"X3DObject")
				{
					CMeshBuilder cm;	//dummy, not used
					X3DObject cloneObj;
									
					for(int i=0;i <l_dupcount;i++)	{					
						//oRoot.AddPrimitive(L"EmptyPolygonMesh", clonethis.GetName(),cloneObj);
						oRoot.AddPolygonMesh(clonethis.GetName(),cloneObj,cm);			//add mesh to scene			
						Operator CopyOp = xsifactory.CreateObjectFromPreset(L"CopyOp",L"Operators");	//get copy op
	
						if ( CopyOp.Connect(clonethis.GetActivePrimitive().GetRef().GetAsText()+L";"+cloneObj.GetActivePrimitive().GetRef().GetAsText()) != CStatus::OK) 
						{
							logmsg(L"Could not copy object " + clonethis.GetActivePrimitive().GetRef().GetAsText()+L";"+cloneObj.GetActivePrimitive().GetRef().GetAsText() );
							return CStatus::Fail;
						}
						Material mat = clonethis.GetMaterial();
						cloneObj.SetMaterial(clonethis.GetMaterial());
						l_dupclones.Add(cloneObj);
					}
				}
			}
			*/
	}

	void getDataFromViewSamples(ToolContext &in_ctxt, CVector3Array &l_samples, vector<double> &l_samplesFinal, vector<LONG> &l_samplePolyInd, CRefArray &l_samplesObjRefs, CVector3 &l_mousecentersample)
	{
		// Get position of sample in scene (raycast polymeshes / position on grid)
		for (LONG i = 0; i < l_samples.GetCount();i++)	{
			CVector3 l_sampleInView;
			in_ctxt.WorldToView(l_samples[i], l_sampleInView);
			LONG samplePosX = static_cast<LONG>(l_sampleInView[0]);
			LONG samplePosY = static_cast<LONG>(l_sampleInView[1]);
			CRef sampleObjRef;
			if (m_options.stackclones == 0 || m_lineDrawing)	//line drawing forces stack clones OFF
				sampleObjRef = m_pickbufferObj.GetObjectAtPosition(samplePosX, samplePosY );
			else	{
				CRef objmouse = m_pickbufferObj.GetObjectAtPosition(samplePosX, samplePosY );
				CRefArray l_objects;
				for (LONG i=0;i < m_strokes.size();i++)	{				
						l_objects += m_strokes[i].strokeclones;
				}
				if ( objmouse.IsValid() )
					l_objects.Add(objmouse);

				PickBuffer l_pickbuff = in_ctxt.GetPickBuffer(samplePosX,samplePosY,8,8, siObjectFilter, l_objects);
				sampleObjRef = l_pickbuff.GetObjectAtPosition(samplePosX, samplePosY );
			}
			if ( sampleObjRef.IsValid() && sampleObjRef.GetClassIDName() == L"X3DObject")	// there is an object under cursor
			{
			
				X3DObject x3d_sampleobj (sampleObjRef);	
				CVector3 samplePos;
				LONG samplePolyInd;
				LONG faceIndex;	

				if ( m_options.stackclones == 0 || (m_options.stackclones && m_toolMode == kDrawingDrop) || m_lineDrawing)
					faceIndex = m_pickbuffer.GetComponentIndexAtPosition(samplePosX, samplePosY );
				else	{
					CRefArray l_objects; 
					l_objects.Add(sampleObjRef);
					PickBuffer l_pickbuffComp = in_ctxt.GetPickBuffer(samplePosX,samplePosY,8,8, siPolygonFilter, l_objects);
					faceIndex = l_pickbuffComp.GetComponentIndexAtPosition(samplePosX, samplePosY );
				}

				if (faceIndex != -1)
				{	
					CStatus raycaststatus = GetSinglePolyLocationSample2( in_ctxt, x3d_sampleobj, l_samples[i], faceIndex, samplePos, samplePolyInd);
					
					if (raycaststatus == CStatus::OK)	{
						if (i != (l_samples.GetCount() - 1 ) )	{
							l_samplesFinal.push_back(samplePos[0]);
							l_samplesFinal.push_back(samplePos[1]);
							l_samplesFinal.push_back(samplePos[2]);
							l_samplePolyInd.push_back( samplePolyInd );
							l_samplesObjRefs.Add(sampleObjRef);
						}
						else	{
							l_mousecentersample = samplePos;
						}
						
					}
					else	{
#ifdef _DEBUG
						logmsg(L" RAYCAST FAIL! if (raycaststatus == CStatus::OK)");
#endif
					}
					
				}
			}
			// *** sample on grid
			else	
				if ( m_options.gridpaint )
				{		
					CPlane gridPlane;	
					in_ctxt.GetGridPlane(gridPlane);	
					m_pickedNormal = gridPlane.GetNormal();
					in_ctxt.SetManipulationPlane(gridPlane);
					CVector3 samplePos;
					if ( in_ctxt.GetWorldPosition(samplePosX,samplePosY, samplePos) == CStatus::OK)	
					{
							if (i != (l_samples.GetCount() - 1) )	{
								l_samplesFinal.push_back(samplePos[0]);
								l_samplesFinal.push_back(samplePos[1]);
								l_samplesFinal.push_back(samplePos[2]);
								l_samplePolyInd.push_back( -1 );
								l_samplesObjRefs.Add( CRef() );
							}
							else	{
								l_mousecentersample = samplePos;
							}
					}
					else 
						logmsg(L"grid get world pos. fail");
					in_ctxt.ClearManipulationPlane();
				}
		}
		//pop last sample (brush center)
	}
	
	CStatus getDataUnderCursor(ToolContext &in_ctxt,bool in_gridpicked, LONG x, LONG y, CVector3 &rx,CVector3 &ry,CVector3 &rz, CTransformation &localToGlobal)
	{
		X3DObject l_picked;
		PolygonFace pface;
		if (!in_gridpicked)	
		{
			if ( m_options.stackclones == 0 || (m_options.stackclones && m_toolMode == kDrawingDrop) || m_lineDrawing)	{
				CRef objmouse = m_pickbufferObj.GetObjectAtPosition(x, y );
				l_picked = objmouse;

				PolygonMesh mmesh = l_picked.GetActivePrimitive().GetGeometry();
				LONG pti = m_pickbuffer.GetComponentIndexAtPosition(x, y);

				m_pickedPolyFace = pface = mmesh.GetPolygons().GetItem(pti);
				m_picked = l_picked;
			}
			else	
			{
				CRef objAtmouse = m_pickbufferObj.GetObjectAtPosition(x, y );
				CRefArray l_objects;
				for (LONG i=0;i<m_strokes.size();i++)
					l_objects += m_strokes[i].strokeclones;
				l_objects.Add(objAtmouse);
				PickBuffer l_pickbuff = in_ctxt.GetPickBuffer(x,y,4,4, siObjectFilter, l_objects);
				objAtmouse = l_pickbuff.GetObjectAtPosition(x, y );
				if ( !objAtmouse.IsValid() )
					return CStatus::False;
				l_picked = objAtmouse;
				PolygonMesh mmesh = l_picked.GetActivePrimitive().GetGeometry();
				CRefArray l_cobjects; 
				l_cobjects.Add(objAtmouse);
				PickBuffer l_pickbuffComp = in_ctxt.GetPickBuffer(x,y,4,4, siPolygonFilter, l_cobjects);
				LONG pti = l_pickbuffComp.GetComponentIndexAtPosition(x, y);
				if (pti == -1)	{
						#ifdef _DEBUG
							logmsg(" l_pickbuffComp.GetComponentIndexAtPosition(x, y); FAIL");
						#endif
						return CStatus::False;
				}
				pface = mmesh.GetPolygons().GetItem(pti);
			}

			localToGlobal = l_picked.GetKinematics().GetGlobal().GetTransform();	
			//set brush rotation
			if (!m_screenspacebrush)
				AlignPrimitiveToComponentRefFrame(in_ctxt, pface,l_picked,rx,ry,rz,false,false);
			else	{			
				CPlane cp;
				in_ctxt.GetViewPlane(cp);
				rx = cp.GetTangentU();
				rz = cp.GetTangentV();
				ry = cp.GetNormal();		
				rx.Cross(ry,rz);
			}
		}
		//get position from GRID
		else	{	
				m_pickedPolyFace = PolygonFace();
				CPlane gridPlane;			
				in_ctxt.GetGridPlane(gridPlane);
				in_ctxt.SetManipulationPlane(gridPlane);			
				if ( in_ctxt.GetWorldPosition(x,y,m_pickedSubcompPos) != CStatus::OK)
					return CStatus::OK;
				in_ctxt.ClearManipulationPlane();			
				CVector3 cameraPoint;
				if ( in_ctxt.WorldToCamera( m_pickedSubcompPos, cameraPoint ) != CStatus::OK || cameraPoint.GetZ() > 0.0 )
					return CStatus::OK;		
				//set brush rotation
				rx = gridPlane.GetTangentU();	
				m_pickedNormal = ry = gridPlane.GetNormal();			
				rz = gridPlane.GetTangentV();
		}
		return CStatus::OK;
	}
	CString RemoveOverlaps(ToolContext &in_ctxt, LONG startInd, LONG endInd)
	{
		CString deletionString = L"";
		for (LONG i=startInd; i < endInd; i++)
		{
			m_strokes[ i ].flag.clear();
			for (LONG j=0; j< m_strokes[ i ].dupCount; j++)
			{
				CVector3 bboxminA;
				CVector3 bboxmaxA;
				CVector3 t;
				bool deletion_flag = false;
			
				X3DObject srcobj = m_strokes[ i ].strokeSourceObj;
				if ( srcobj.GetType() == L"polymsh" ||  srcobj.GetType() == L"#model" )
				{
					CTransformation c_t =  m_strokes[ i ].transform[ j ];
					c_t.AddLocalScaling( CVector3(m_options.BBoxScale,m_options.BBoxScale,m_options.BBoxScale ) );
					if (srcobj.GetType() == L"#model" )
						GetModelBoudingBox( srcobj, bboxminA, bboxmaxA, t, true, c_t);
					else
						GetBoudingBox( srcobj, bboxminA, bboxmaxA, t, true, c_t);
		
					for( LONG ii=0; ii < m_strokes.size(); ii++)	
					{
						for (LONG jj=0; jj < m_strokes[ ii ].transform.size(); jj++)
						{
							if (jj == j && i == ii )	//dont test with itself
								continue;
								
							CVector3 bboxminB, bboxmaxB;
							X3DObject l_obj;
							if (ii == i)
								l_obj = srcobj;
							else
								l_obj = m_strokes [ ii ].strokeSourceObj;
							CTransformation c_t =  m_strokes[ ii ].transform[ jj ];
							c_t.AddLocalScaling( CVector3(m_options.BBoxScale,m_options.BBoxScale,m_options.BBoxScale ) );
							if ( l_obj.GetType() == L"polymsh" || srcobj.GetType() == L"#model"  )
							{
								if (srcobj.GetType() == L"#model" )
								{
									if ( GetModelBoudingBox( l_obj,bboxminB, bboxmaxB,  t,true, c_t) != CStatus::OK)
										continue;
								}
								else
									GetBoudingBox( l_obj,bboxminB, bboxmaxB,  t,true, c_t );

								if ( bboxIntersect( bboxminA, bboxmaxA, bboxminB, bboxmaxB ) )
								{
									deletion_flag = true;
									break;
								}
							}
							
						}
						if(deletion_flag)
							break;
					}	//for( LONG ii=0; ii < m_strokes.size(); ii++)	
				}	//if ( clonethis.GetType() == L"polymsh" )	
				m_strokes[ i ].flag.push_back( deletion_flag );
			} //end intersection test
#ifdef _DEBUG
			logmsg(L"stroke flag cnt : " + CString( m_strokes[ i ].flag.size() ) );
			logmsg(L"m_strokes[ i ].strokeclones.GetCount() flag cnt : " + CString( m_strokes[ i ].strokeclones.GetCount() ) );
			logmsg(L"m_strokes[ i ].dupCount  : " + CString( m_strokes[ i ].dupCount ) );			
			
#endif|
			assert(m_strokes[ i ].flag.size() == m_strokes[ i ].dupCount );
			LONG j = 0;
			LONG counter = 0;
			while (j < m_strokes[ i ].flag.size() )
			{
				if  ( m_strokes[ i ].flag[ j ] )
				{
					s_stroke *my_stroke = &m_strokes[ i ];
					my_stroke->dupCount--;
					
					my_stroke->flag.erase( my_stroke->flag.begin()+j );
					my_stroke->transform.erase( my_stroke->transform.begin() + j );
					my_stroke->rotx.erase( my_stroke->rotx.begin() + j );
					my_stroke->roty.erase( my_stroke->roty.begin() + j );
					my_stroke->rotz.erase( my_stroke->rotz.begin() + j );
					my_stroke->rotrand.erase( my_stroke->rotrand.begin() + j );
					my_stroke->transrand.erase( my_stroke->transrand.begin() + j );
					my_stroke->sclrand.erase( my_stroke->sclrand.begin() + j );
					my_stroke->posOffset.erase( my_stroke->posOffset.begin() + j );

					deletionString += my_stroke->strokeclones[ j ].GetAsText();
					CRef obj = my_stroke->strokeclones[ j ];
					my_stroke->strokeclones.Remove( obj );
#ifdef _DEBUG
					logmsg( deletionString );
#endif
					if (deletionString != L"")
						deletionString += L",";
					logmsg(L"counter: ", counter);
					logmsg(L"j:", j );
					X3DObject x3dobj( my_stroke->strokeclones[ counter ] );
					in_ctxt.EndKinematicStateUpdate( x3dobj.GetKinematics().GetGlobal() );			
#ifdef _DEBUG
					cntb++;
#endif
				}
				else
					j++;
				counter++;
			}
		}
		// ************** REMOVE EMPTY m_stroke[ ? ]
		vector<s_stroke>::iterator it = m_strokes.begin();
		for( it = m_strokes.begin(); it != m_strokes.end(); )
			if ( !it->transform.size() )	{
				it->strokeSourceObj = CRef();
				it->strokeclones = CRefArray();
				it = m_strokes.erase( it );				
			}
			else
				++it;
		return deletionString;
	}

	CStatus UpdateMouseDragLine(ToolContext &in_ctxt, LONG sx, LONG sy, LONG endx, LONG endy, bool interpolateMode)
	{
		// Number of 'strokes' in line (line length / spacing)
		CVector3 distance, endPos,startPos;
		in_ctxt.EnableSnapping( true );
		in_ctxt.GetWorldPosition(sx,sy,startPos);
		in_ctxt.GetWorldPosition(endx,endy,endPos);				
		in_ctxt.EnableSnapping( false );
		distance = endPos;
		distance -= startPos;		
		LONG strokeCount = (LONG)(distance.GetLength() / m_options.builddist);
		
		clock_t timer = clock();
		srand( m_options.randseed + clock() );	//seed random number
						
		if ( !strokeCount )
			return CStatus::OK;
		// ******** Delete overlaps
		
		if ( !interpolateMode && m_options.DeleteOverlaps )	{
			//if (strokeCount < m_strokes.size() && strokeCount > 1)	
			if (1)
			{
				CString sdeletethis = L"";
				sdeletethis = RemoveOverlaps(in_ctxt, 0, (LONG)m_strokes.size());;
				if (sdeletethis != L"")	{
						CValue retval;
						CValueArray args(1);
						args[0] = sdeletethis;
						Application().ExecuteCommand( L"DeleteObj", args, retval );
				}			
			}
		}
		
		// ******** Delete duplicates
		if (!interpolateMode)
		{
			if (strokeCount < m_strokes.size() && strokeCount > 1)	
			{
				CString sdeletethis = L"";
				while (m_strokes.size() > strokeCount)
				{
					LONG ind = (LONG)m_strokes.size() - 1;
					// *********** End Kine state update
					for( int i=0; i < m_strokes [ind].strokeclones.GetCount(); i++)	{
						X3DObject x3dobj(m_strokes[ ind ].strokeclones[ i ] );
						in_ctxt.EndKinematicStateUpdate( x3dobj.GetKinematics().GetGlobal() );
#ifdef _DEBUG
								cntb++;
#endif
					}
					// get deletion string
					CString sdeletestroke = m_strokes[ ind ].strokeclones.GetAsText();
					if (sdeletethis != L"" && sdeletestroke != L"")
						sdeletethis += ",";
					sdeletethis += sdeletestroke;		
					// clean ref arrays
					m_strokes[ ind ].strokeSourceObj = CRef();
					m_strokes[ ind ].strokeclones = CRefArray();
					m_strokes.pop_back();
				}
				if (sdeletethis != L"")	{
					CValue retval;
					CValueArray args(1);
					args[0] = sdeletethis;
					Application().ExecuteCommand( L"DeleteObj", args, retval );
				}
			}
		}
		
		LONG lastStrokeInd = static_cast<LONG>( m_strokes.size() );
		// ******** Add new duplicates **********
		if ( strokeCount > m_strokes.size() || interpolateMode )	
		{
			LONG js = interpolateMode ? 0 : (LONG)m_strokes.size();
			LONG je = interpolateMode ? strokeCount : strokeCount;

			for (LONG j = js; j < je; j++)
			{
				bool gridpicked;
				double t = (double)(j+1) / (double)strokeCount;
						
				LONG x = lerp_value(sx,endx,t );
				LONG y = lerp_value(sy,endy,t );	
				CVector3 inter = startPos.LinearlyInterpolate( startPos, endPos, t);
				gridOrObjectUnderMouseCursor(in_ctxt, x, y, inter, gridpicked);

				m_pickedSubcompPos = inter;
				
				UpdateMouseDrag(in_ctxt, x,  y, gridpicked, false, true, inter);

				m_dragtime = 0;						
				m_draglastpickedPos = inter;
			}

			CLongArray l_paintClonesDupCount;
			for (LONG k=0;k < m_paintClones.GetCount(); k++)
				l_paintClonesDupCount.Add( 0 );

			for (LONG i= lastStrokeInd; i < m_strokes.size(); i++)
			{
				for (LONG k=0;k < m_paintClones.GetCount(); k++)
					if (m_paintClones[k] == m_strokes[ i ].strokeSourceObj)	
					{
						l_paintClonesDupCount[ k ] += m_strokes[ i ].dupCount;
						break;
					}		
			}

			// ************* Fill Cref arrays in m_strokes
			for (LONG k=0;k < m_paintClones.GetCount(); k++)	
				if ( l_paintClonesDupCount[ k ] )
				{
					CRefArray l_dupclones;
#ifdef _DEBUG
					logmsg(L"cloe dup count",  l_paintClonesDupCount[ k ] );
#endif
					Duplicate( m_paintClones [ k ], l_paintClonesDupCount[ k ], m_options.paintinstances, l_dupclones );
					
					// Add clone to group
					if ( m_options.groupName != L"" && m_cloneGroup.IsValid() )	
						for(LONG ii=0; ii < l_dupclones.GetCount(); ii++)
							m_cloneGroup.AddMember( l_dupclones[ ii ] );
					// Add clones to Null
					if ( m_options.nullName != L"" && m_cloneNullParent.IsValid() )	
							m_cloneNullParent.AddChild( l_dupclones );
					else
						Application().GetActiveSceneRoot().AddChild( l_dupclones );
					// ********** Begin updating kinematics
					LONG j = 0;				
					for (LONG i= lastStrokeInd; i < m_strokes.size(); i++)
					{					
						if (m_paintClones[k] == m_strokes[ i ].strokeSourceObj)	
						{
							for (LONG z = 0; z < m_strokes[ i ].transform.size(); z++)
							{				
								X3DObject xobj( l_dupclones[ j ] );
								KinematicState kstateobj( xobj.GetKinematics().GetGlobal() );
								in_ctxt.BeginKinematicStateUpdate( kstateobj );
#ifdef _DEBUG
								cnta++;
#endif
								//in_ctxt.UpdateKinematicState( kstateobj, m_strokes[ i ].transform[ z ] );
								// *********** Add reference
								m_strokes[ i ].strokeclones.Add( l_dupclones[ j ] );
								j++;
							}
						}
						if (j+1 > l_dupclones.GetCount() )
							break;
					}
					
					l_dupclones.Clear();
				}
#ifdef _DEBUG
				for (LONG k=0;k < m_strokes.size(); k++)	{
					s_stroke *cs = &m_strokes[k];
					LONG cnt = cs->strokeclones.GetCount();
					logmsg(L"************* stroke cnt: ", cnt);
					logmsg(L"************* cs->transform.size()  cnt: ", cs->transform.size() );
					logmsg(L"*************  cnt: cs->rotz.size()", cs->rotz.size());
						logmsg(L"*************  cnt: cs->rotz.size()", cs->rotz.size());
							logmsg(L"*************  cs->rotrand.size()  cnt: ",  cs->rotrand.size() );
								logmsg(L"************* cs->dupCount  cnt: ", cs->dupCount );

					assert( cnt == cs->transform.size() );
					assert( cnt == cs->dupCount );
					assert( cnt == cs->rotrand.size() );
					assert( cnt == cs->rotz.size());
					assert( cnt == cs->transrand.size());
				}
#endif
		}
		LONG startindex = interpolateMode ? lastStrokeInd - 1: 0;
		
		// ************** update stroke transforms
		if (!interpolateMode)
		{
			
			const LONG bdivisions = static_cast<LONG>( m_strokes.size() ) * 10;
			LONG maxPoint = bdivisions + 1;
			CVector3 bpreviousPoint;
			CVector3 a,b,c,d;
			a=CVector3(sx,sy,0);
			d=CVector3(endx, endy,0);
			vector<double> arcLenghts;
			if ( m_mouseBezierX != endx && m_mouseBezierY != endy )
				b=c=CVector3( m_mouseBezierX, m_mouseBezierY,0);
			else
				b=c=b.LinearlyInterpolate(a,d,.5);
			bezier(bpreviousPoint, a,b,c,d,0);
			// ********************
			// calculate arc lengths
			// and store them in vectorarray arcLenghts
			// ********************
			double bsum = 0.0;
			arcLenghts.push_back(0.0);
			for (LONG i=1; i <= maxPoint; i++)	{
				CVector3 p;
				bezier (p, a,b,c,d, i / (double)maxPoint);
				double pa = p.GetX() - bpreviousPoint.GetX();
				double pb = p.GetY() - bpreviousPoint.GetY();
				bsum += sqrt( pa*pa + pb*pb );
				arcLenghts.push_back( bsum );
				bpreviousPoint = p;
			}
			
			for (LONG j = startindex; j < m_strokes.size(); j++)
			{
				bool gridpicked;
				double t;
				if ( m_strokes.size() != 1 )
					t = (double)(j) / ( (double)(m_strokes.size()-1)  );
				else 
					t = 0.0;			
				LONG x,y;
				CVector3 l_pos;
				CVector3 dest;
				// calculate t with help of arclengths
				if ( arcLenghts.size() )	
				{
					double u = t;
					double targetArcLength = u * arcLenghts[ arcLenghts.size() - 1];
					
					LONG index = 0;
					if (targetArcLength)
						while (targetArcLength >= arcLenghts[ index ] )
							if (index < (arcLenghts.size()-1) )
								index++;
							else 
								break;
					
					if ( arcLenghts[ index ] == targetArcLength )
						t = index / (double) (arcLenghts.size() - 1);
					else
					{
						double lengthBefore = arcLenghts[ index ];
						double lengthAfter = arcLenghts[ index+1 ];
						double segmentLength = lengthAfter - lengthBefore;
						double segmentFraction = (targetArcLength - lengthBefore) / segmentLength;

						t = (index + segmentFraction) / (double) (arcLenghts.size() - 1 );
					}
				}

				bezier(dest, a,b,c,d,t);
				x = static_cast<LONG>( dest.GetX() );
				y = static_cast<LONG>( dest.GetY() );
				if ( gridOrObjectUnderMouseCursor(in_ctxt, x, y, l_pos, gridpicked) != CStatus::OK)
					continue;
			
				m_pickedSubcompPos = l_pos;
				CVector3Array l_samples;
				for( LONG i=0; i < m_strokes[ j ].strokeclones.GetCount(); i++ )	
				{
					CVector3 diff = l_pos;
					diff += m_strokes[ j ].posOffset[ i ];
					l_samples.Add( diff );	
				}
				l_samples.Add( l_pos );
				vector<double>	l_samplesFinal;	
				vector<LONG>	l_samplePolyInd;
				CRefArray		l_samplesObjRefs;	
				CVector3		l_mousecentersample;
				getDataFromViewSamples(in_ctxt, l_samples, l_samplesFinal, l_samplePolyInd, l_samplesObjRefs, l_mousecentersample );
				m_strokes[ j ].strokeCenter = l_mousecentersample;	//this stroke center position
				LONG ind = 0;
				for(LONG i=0;i < (LONG)l_samplesFinal.size();i += 3)	
				{
					CTransformation transform = m_strokes[ j ].transform[ ind ];
					if ( ind > l_samplePolyInd.size() && l_samplePolyInd[ ind ] != -1 )			
						continue;
					X3DObject l_sampleObj;
					CPolygonFaceRefArray meshPolys;

					if( l_samplePolyInd[ ind ] != -1 )	// sample not on grid
					{
						l_sampleObj = l_samplesObjRefs[ ind ];
						PolygonMesh pmesh = l_sampleObj.GetActivePrimitive().GetGeometry();
						meshPolys =  pmesh.GetPolygons();
					}
				
					PolygonFace sampleFace;
					CVector3 rx,ry,rz;
			
					// ********************** Set rotation *******************
					CRotation myrot, offsetrot;
				
					if ( l_samplePolyInd[ ind ] != -1 && m_options.Align )	{
						sampleFace = meshPolys[ l_samplePolyInd[ ind ] ];			
						AlignPrimitiveToComponentRefFrame(in_ctxt,sampleFace,l_sampleObj,rx,ry,rz,false,true);
					}
					else	{
						CPlane gridPlane;			
						in_ctxt.GetGridPlane(gridPlane);	
						rx = gridPlane.GetTangentU();
						ry = gridPlane.GetNormal();
						rz = gridPlane.GetTangentV();
						rx.Cross(ry,rz);
					}
					m_strokes[ j ].rotx[ ind ] = rx;
					m_strokes[ j ].roty[ ind ] = ry;
					m_strokes[ j ].rotz[ ind ] = rz;
			
					CVector3 temp_ry;	
					// change axis of rotation
					switch (m_options.alignaxis)	{	
						case ky:
							if (m_options.AlignAxisFlip )
								ry.NegateInPlace();
							break;
						case kx:
							temp_ry = ry;
							if (m_options.AlignAxisFlip)
								rx.NegateInPlace();
							ry = rx;
							rx = temp_ry;
							rz.Cross(rx,ry);
							break;
						case kz:
							temp_ry = ry;
							if (m_options.AlignAxisFlip)
								rz.NegateInPlace();
							ry = rz;
							rz = temp_ry;
							rx.Cross(ry,rz);
							break;
					}
					// calc & store & apply random/offset rotations
					myrot.SetFromXYZAxes(rx,ry,rz);
					CRotation randomRot;
					CQuaternion frot = ApplyRotation(m_strokes[ j ].rotrand[ ind ], myrot);
					myrot.SetFromQuaternion( frot );
					transform.SetRotation(myrot);
					// ************** Set position **************
					CVector3 samplePos(l_samplesFinal[i],l_samplesFinal[i+1],l_samplesFinal[i+2]);
					CVector3 *randPos = &m_strokes[ j ].transrand[ ind ];
					CVector3 axisoffset (m_options.axisoffsetx, m_options.axisoffsety, m_options.axisoffsetz );
					CVector3 v_pos = samplePos;
					if (m_options.randlocalTrans == 0)
						v_pos += *randPos;
					if (m_options.localTrans == 0)
						v_pos += axisoffset;
					transform.SetTranslation( v_pos );
					CVector3 localTransf(0,0,0);
					if ( m_options.randlocalTrans != 0) 
						localTransf = *randPos;
					if ( m_options.localTrans != 0 ) 
						localTransf += axisoffset;
					if ( m_options.randlocalTrans != 0 || m_options.localTrans != 0)
						transform.AddLocalTranslation( localTransf );
					// ******************************************
					m_strokes[ j ].transform[ ind ] = transform;	
					calcScaleRampAndFollowStroke(in_ctxt);
				
					ind++;
				}
		
			}
		}
		if ( m_options.scaleramp )
			startindex = 0;
		// ************** Update KineState
		for (LONG i=startindex; i< m_strokes.size() ; i++)
			for(LONG j=0; j< m_strokes[i].strokeclones.GetCount(); j++)	
			{		
				X3DObject xobj( m_strokes[i].strokeclones[j] );
				KinematicState kstateobj( xobj.GetKinematics().GetGlobal() );
				in_ctxt.UpdateKinematicState( kstateobj, m_strokes[ i ].transform[ j ] );
			}
		
		return CStatus::OK;
	}
	void calcScaleRampAndFollowStroke(ToolContext &in_ctxt)
	{
			if (  m_options.scaleramp != 0 || m_options.FollowStroke )	
			{
				//ramp scale
				double scx=0,scy=0,scz=0,mscx=0,mscy=0,mscz=0;
				if ( m_options.scaleramp != 0 )	{
					if ( m_options.scalerampuni != 0 )	{
						scx = scy = scz = m_options.scalex;
						mscx = mscy = mscz = m_options.scaleminx;
					}
					else	{
						scx = m_options.scalex;
						scy = m_options.scaley;
						scz = m_options.scalez;
						mscx = m_options.scaleminx;
						mscy = m_options.scaleminy;
						mscz = m_options.scaleminz;
					}
				}

				// calculate smoothed mouse positions
				CVector3Array smoothCenters;
				if ( m_options.FollowStroke )	{
					for(LONG j=0;j < m_strokes.size(); j++)	
					{	
						double len=0;
						LONG ind = 0;
						CVector3 avgpos;
						while ( len < m_options.StrokeSmoothLength && j+ind <  m_strokes.size() )	{
							LONG counter = j+ind;
							CVector3 v;
							CVector3 vw,vwn;
							
							if (counter > 0)	{
								v =  m_strokes[counter-1].strokeCenter;
								v -= m_strokes[counter].strokeCenter;	
							}
							avgpos += m_strokes[counter].strokeCenter;
							
							len += v.GetLength();		
							ind+=1;
						}
						if (ind == 0)
							avgpos = m_strokes[j].strokeCenter;	
						else
							avgpos *= (1.0/ (double)ind);
						smoothCenters.Add(avgpos);
					}
				}

				//calculate follow stroke rotation
				for(LONG j=0;j < m_strokes.size(); j++)	
				{	
					double t = j / (double)m_strokes.size();

					double scale_x,scale_y,scale_z;
					if ( m_options.scaleramp != 0 )	{ 
						scale_x = lerp_value(mscx,scx,t);
						scale_y = lerp_value(mscx,scy,t);
						scale_z = lerp_value(mscx,scz,t);
					}
					LONG l_strokeCloneCnt = static_cast<LONG>( m_strokes[j].transform.size() );
					for(LONG i=0;i < l_strokeCloneCnt;i++)
					{
						CTransformation kt		= m_strokes[j].transform[ i ];
						CQuaternion		old_rot = kt.GetRotationQuaternion();			
						// *********** Apply scale
						if (m_options.scaleramp != 0 )	{
							CVector3 sclrand = m_strokes[ j ].sclrand[ i ];	
							kt.SetScalingFromValues( scale_x+sclrand[0],scale_y+sclrand[1],scale_z+sclrand[2] );	// ramp scale
						}
						bool update_rotation = false;
						// ************ Calculate follow stroke
						if ( m_options.FollowStroke )	
						{			// stroke alignment
							CVector3 fpos2;
							
							const double l_scalevect = 1;
							if (j)	{
							
								fpos2 =  smoothCenters[j-1];
								fpos2 -= smoothCenters[j];			
							
								if ( (j + 1 )< m_strokes.size())	
								{
									CVector3 fpos3;
									fpos3 = smoothCenters[j];
									fpos3-= smoothCenters[j+1];
									fpos2 = lerp_value(fpos3, fpos2,.5);
								}
							
							}
							else	{
								fpos2 = smoothCenters[0];							
								fpos2 -= smoothCenters[1];
							}
														
							if (m_lineDrawing)	{
								fpos2 *= 100000.0;		// hmm.temporary

							}
							fpos2.NormalizeInPlace();

							CVector3 ax,ay,az;	//out rotation vectors
							if (m_options.AlignAxis == kx)	{	
						
								az= m_strokes[j].roty[i];
								ay = fpos2;	
								if(m_options.AlignAxisFlip == 0)
									ay.NegateInPlace();
								az.NormalizeInPlace();
								ax.Cross(ay,az);
								az.Cross(ax,ay);	
							}
							else	
								if (m_options.AlignAxis == ky)	{
									ax = fpos2;
									ay = m_strokes[j].roty[i];
									if(m_options.AlignAxisFlip != 0)
										ay.NegateInPlace();
									az.Cross(ax,ay);
									ax.Cross(ay,az);						
								}
								else
								if (m_options.AlignAxis == kz){
									az= fpos2;
									ax = m_strokes[j].roty[i];
									ay.Cross(ax,az);
									if(m_options.AlignAxisFlip != 0)
										ay.NegateInPlace();
									ax.Cross(ay,az);
									az.Cross(ax,ay);
								}
							az.NormalizeInPlace();
							ax.NormalizeInPlace();
							CRotation new_rot;	
							new_rot.SetFromXYZAxes(ax,ay,az);
							new_rot.SetFromQuaternion( ApplyRotation( m_strokes[j].rotrand[ i ], new_rot) );
							kt.SetRotation( new_rot );					
						}	// ------ if ( m_options.FollowStroke )	
						m_strokes[j].transform[ i ] = kt;
					}	// ------ for(LONG i=0;i < l_strokeCloneCnt;i++)
				}
			}	// if (  m_options.scaleramp != 0 || m_options.FollowStroke )		
	}

	CStatus UpdateMouseDrag(ToolContext &in_ctxt, LONG x, LONG y, bool l_gridpicking, bool l_duplicate, bool use_inputPos, CVector3 inputPos)
	{
#ifdef _DEBUG
		logmsg("UpdateMouseDrag called");
#endif
		Model oRoot			= Application().GetActiveSceneRoot();
		LONG paintinstances = m_options.paintinstances;
		LONG cloneCount		= m_options.cloneCount;
		//get values from ppg
		
	
		PolygonFace pface;

		CVector3 l_gridu, l_gridv, l_gridnormal; //for samples laying on grid

		CTransformation localToGlobal;
		CVector3 rx,ry,rz;
		clock_t timer = clock();			
		getDataUnderCursor( in_ctxt, l_gridpicking, x, y, rx, ry, rz, localToGlobal);	// get stroke rotation....pretty reduntad stuff here...needs work
		CVector3 lpos = m_pickedSubcompPos;	
		if (use_inputPos)
			lpos = inputPos;
		double scale =  m_brush.GetRadius();
		// drop always in center
		if ( m_toolMode == kDrawingDrop)	{
			cloneCount = 1;
			scale = 0.0;
		}
		// BUILD SAMPLES for clones placement
		srand( m_options.randseed );	//seed random number
		
		CVector3Array l_samples, l_centersample;
		BuildCircleSamples(in_ctxt, scale, cloneCount, lpos,rx,rz,ry, l_samples);
		BuildCircleSamples(in_ctxt, 0.0, 1, lpos,rx,rz,ry,			  l_centersample);		// sample in center of brush/mouse position/
		l_samples.Add( l_centersample[0] );

		vector<double>	l_samplesFinal;	
		vector<LONG>	l_samplePolyInd;
		CRefArray		l_samplesObjRefs;	
		CVector3		l_mousecentersample;
		
		// Get position of sample in scene (raycast polymeshes / position on grid)
		getDataFromViewSamples(in_ctxt, l_samples, l_samplesFinal, l_samplePolyInd, l_samplesObjRefs, l_mousecentersample );

		LONG l_dupcount = (LONG)l_samplesFinal.size() / 3;

		if (l_dupcount == 0)	//no samples...raycast fail?
			if (m_toolMode == kDrawingDrop)
				return CStatus::Fail;
			else
				return CStatus::OK;
		//*********fill data for drop manipulation
		if ( m_pickedPolyFace.IsValid() )
			AlignPrimitiveToComponentRefFrame(in_ctxt,m_pickedPolyFace,m_picked, m_pickedFrameAxisX,m_pickedFrameAxisY,m_pickedFrameAxisZ,false,true);

		if (m_options.Align == 0 || !m_pickedPolyFace.IsValid() )	//no poly under cursor or surface alignment is off
		{
				CPlane gridPlane;			
				in_ctxt.GetGridPlane(gridPlane);	
				m_pickedFrameAxisX = gridPlane.GetTangentU();
				m_pickedFrameAxisY = gridPlane.GetNormal();
				m_pickedFrameAxisZ = gridPlane.GetTangentV();
				m_pickedFrameAxisX.Cross( m_pickedFrameAxisY,m_pickedFrameAxisZ );
		}
		CVector3 temp_ry;
		if ( m_toolMode == kDrawing )		//????????????????????????????????????????????????????????
		switch (m_options.alignaxis)	{	
				case ky:
					if (m_options.AlignAxisFlip )
						m_pickedFrameAxisY.NegateInPlace();
					break;
				case kx:
					temp_ry = m_pickedFrameAxisY;
					if (m_options.AlignAxisFlip)
						m_pickedFrameAxisX.NegateInPlace();
					m_pickedFrameAxisY = m_pickedFrameAxisX;
					m_pickedFrameAxisX = temp_ry;
					m_pickedFrameAxisZ.Cross(m_pickedFrameAxisX,m_pickedFrameAxisY);
					break;
				case kz:
					temp_ry = ry;
					if (m_options.AlignAxisFlip)
						m_pickedFrameAxisZ.NegateInPlace();
					m_pickedFrameAxisY = m_pickedFrameAxisZ;
					m_pickedFrameAxisZ = temp_ry;
					m_pickedFrameAxisX.Cross(m_pickedFrameAxisY,m_pickedFrameAxisZ);
					break;
			}

		//*****************
	
		Factory xsifactory = Application().GetFactory();	
				
		// clone random or cycle through list  ***
		LONG prev_cloneind = m_cloneId;
		if ( m_options.listpick == 0 )
			m_cloneId = rand() % m_paintClones.GetCount();
		else	{
			prev_cloneind = m_cloneId;
			m_cloneId += 1;
			if ( m_cloneId >=  m_paintClones.GetCount() )
				m_cloneId = 0;
		}
		
		X3DObject clonethis = m_paintClones[ m_cloneId ];		// get the x3dobject of source clone
		
		m_strokes.push_back( s_stroke() );		//add new stroke to array
		LONG l_strokeInd = (LONG)m_strokes.size() - 1;	//CURRENT stroke index
		m_strokes[ l_strokeInd ].dupCount = 0;
		m_strokes[ l_strokeInd ].strokeCenter = l_mousecentersample;	//this stroke center position

		LONG ind_cnt = 0;

		for(int i=0;i < (LONG)l_samplesFinal.size();i += 3)	
		{
			if ( ind_cnt > l_samplePolyInd.size() && l_samplePolyInd[ind_cnt] != -1 )
				continue;

			X3DObject l_sampleObj;
			CPolygonFaceRefArray meshPolys;

			if( l_samplePolyInd[ ind_cnt ] != -1 )	// sample not on grid
			{
				l_sampleObj = l_samplesObjRefs[ ind_cnt ];
				PolygonMesh pmesh = l_sampleObj.GetActivePrimitive().GetGeometry();
				meshPolys =  pmesh.GetPolygons();
			}

			CTransformation transform;
			//set scaling	****************************
			if ( m_toolMode != kDrawingDrop )	{
				double sclmodx, sclmody, sclmodz;
				if ( m_options.randscluni != 0 )	{	//uniform scaling
					sclmodx = rand_range(-m_options.randsclx,m_options.randsclx);		
					if ( m_options.randsclrange == 0 )
						sclmodx = abs(sclmodx);	
					sclmody = sclmodx;
					sclmodz = sclmodx;			
				}
				else	
					{
						sclmodx = rand_range(-m_options.randsclx,m_options.randsclx);
						sclmody = rand_range(-m_options.randscly,m_options.randscly);
						sclmodz = rand_range(-m_options.randsclz,m_options.randsclz);
						if ( m_options.randsclrange == 0 )	{
							sclmodx = abs(sclmodx);		
							sclmody = abs(sclmody);		
							sclmodz = abs(sclmodz);
						}							
					}
				m_strokes[ l_strokeInd ].sclrand.push_back( CVector3( sclmodx, sclmody, sclmodz) );		// store randomized scale

				transform.SetScalingFromValues( m_options.scx + sclmodx, m_options.scy + sclmody, m_options.scz+ sclmodz);
			}
			else			// drop paint means we start 0scale
			{
				transform.SetScalingFromValues( 0,0,0 );
			}
			//set position

			CVector3 samplePos(l_samplesFinal[i],l_samplesFinal[i+1],l_samplesFinal[i+2]);
			CVector3 randPos( rand_range(-m_options.randposx,m_options.randposx ),rand_range( -m_options.randposy,m_options.randposy ), rand_range( -m_options.randposz,m_options.randposz ));
			m_strokes[ l_strokeInd ].transrand.push_back( randPos );
		
			//set rotation					
			PolygonFace sampleFace;
			CVector3 rx,ry,rz;
			
			// Set rotation
			CRotation myrot, offsetrot;
				
			if ( l_samplePolyInd[ind_cnt] != -1 && m_options.Align )	{
				sampleFace = meshPolys[ l_samplePolyInd[ ind_cnt ] ];			
				AlignPrimitiveToComponentRefFrame(in_ctxt,sampleFace,l_sampleObj,rx,ry,rz,false,true);
			}
			else	{
				CPlane gridPlane;			
				in_ctxt.GetGridPlane(gridPlane);	
				rx = gridPlane.GetTangentU();
				ry = gridPlane.GetNormal();
				rz = gridPlane.GetTangentV();
				rx.Cross(ry,rz);
			}
			CVector3 temp_ry;
			// store rotation of stroke
			m_strokes[ l_strokeInd ].rotx.push_back(rx);
			m_strokes[ l_strokeInd ].roty.push_back(ry);
			m_strokes[ l_strokeInd ].rotz.push_back(rz);
			
			// change axis of rotation
			switch (m_options.alignaxis)	{	
				case ky:
					if (m_options.AlignAxisFlip )
						ry.NegateInPlace();
					break;
				case kx:
					temp_ry = ry;
					if (m_options.AlignAxisFlip)
						rx.NegateInPlace();
					ry = rx;
					rx = temp_ry;
					rz.Cross(rx,ry);
					break;
				case kz:
					temp_ry = ry;
					if (m_options.AlignAxisFlip)
						rz.NegateInPlace();
					ry = rz;
					rz = temp_ry;
					rx.Cross(ry,rz);
					break;
			}
			// calc & store & apply random/offset rotations
			myrot.SetFromXYZAxes(rx,ry,rz);
			CRotation randomRot;
			randomRot.SetFromXYZAngles( DegreesToRadians( rand_range( - m_options.randrotx,  + m_options.randrotx) ), 
							DegreesToRadians( rand_range( - m_options.randroty,  + m_options.randroty) ),
							DegreesToRadians( rand_range( - m_options.randrotz, + m_options.randrotz) ) );
			m_strokes[ l_strokeInd ].rotrand.push_back( randomRot );
			CQuaternion frot = ApplyRotation(randomRot, myrot);
			myrot.SetFromQuaternion( frot );

			transform.SetRotation(myrot);
			/*
			if (m_options.randlocalTrans != 0)	{
				transform.SetPosX( m_options.axisoffsetx + samplePos[0]);
				transform.SetPosY( m_options.axisoffsety + samplePos[1] );
				transform.SetPosZ( m_options.axisoffsetz + samplePos[2] );
				transform.AddLocalTranslation( CVector3 (	randPos[0], randPos[1], randPos[2]	) );
			}	
			else	{
				transform.SetPosX( m_options.axisoffsetx + samplePos[0]+randPos[0] );
				transform.SetPosY( m_options.axisoffsety  + samplePos[1]+randPos[1] );
				transform.SetPosZ( m_options.axisoffsetz + samplePos[2]+randPos[2] );
			}
			*/
			// ************** Set position **************
			CVector3 axisoffset (m_options.axisoffsetx, m_options.axisoffsety, m_options.axisoffsetz );
			CVector3 v_pos = samplePos;
			if (m_options.randlocalTrans == 0)
				v_pos += randPos;
			if (m_options.localTrans == 0)
				v_pos += axisoffset;
			transform.SetTranslation( v_pos );
			CVector3 localTransf(0,0,0);
			if ( m_options.randlocalTrans != 0) 
				localTransf = randPos;
			if ( m_options.localTrans != 0 ) 
				localTransf += axisoffset;
			if ( m_options.randlocalTrans != 0 || m_options.localTrans != 0)
				transform.AddLocalTranslation( localTransf );
			// ******************************************

			m_strokes[ l_strokeInd ].transform.push_back( transform );
			CVector3  poff = m_strokes[ l_strokeInd ].strokeCenter;
			poff -= CVector3( samplePos[0], samplePos[1], samplePos[2] );
			m_strokes[ l_strokeInd ].posOffset.push_back( poff );
			m_strokes[ l_strokeInd ].strokeSourceObj = clonethis;
			m_strokes[ l_strokeInd ].dupCount++;
			ind_cnt++;	//next sample
		}

		timer = clock();
		// REMOVE OVERLAPS *******************************
		
		if ( m_toolMode != kDrawingDrop )
		{
			if ( m_options.DeleteOverlaps )			// remove unnecessary data from struct
			{
				m_strokes[ l_strokeInd ].flag.clear();
				for (LONG j=0; j < l_dupcount; j++)
				{
					CVector3 bboxminA;
					CVector3 bboxmaxA;
					CVector3 t;
					bool deletion_flag = false;
					if ( clonethis.GetType() == L"polymsh" ||  clonethis.GetType() == L"#model" )
					{
						CTransformation c_t =  m_strokes[ l_strokeInd ].transform[ j ];
						c_t.AddLocalScaling( CVector3(m_options.BBoxScale,m_options.BBoxScale,m_options.BBoxScale ) );
						if (clonethis.GetType() == L"#model" )
							GetModelBoudingBox( clonethis, bboxminA, bboxmaxA, t, true, c_t);
						else
							GetBoudingBox( clonethis, bboxminA, bboxmaxA, t, true, c_t);
		
						for( LONG ii=0; ii < m_strokes.size(); ii++)	
						{
							for (LONG jj=0; jj < m_strokes[ ii ].transform.size(); jj++)
							{
								if (jj == j && l_strokeInd == ii )	//dont test with itself
									continue;
								
								CVector3 bboxminB, bboxmaxB;
								X3DObject l_obj;
								if (ii == l_strokeInd)
									l_obj = clonethis;
								else
									l_obj = m_strokes [ ii ].strokeSourceObj;
								CTransformation c_t =  m_strokes[ ii ].transform[ jj ];
								c_t.AddLocalScaling( CVector3(m_options.BBoxScale,m_options.BBoxScale,m_options.BBoxScale ) );
								if ( l_obj.GetType() == L"polymsh" || clonethis.GetType() == L"#model"  )
								{
									if (clonethis.GetType() == L"#model" )
									{
										if ( GetModelBoudingBox( l_obj,bboxminB, bboxmaxB,  t,true, c_t) != CStatus::OK)
											continue;
									}
									else
										GetBoudingBox( l_obj,bboxminB, bboxmaxB,  t,true, c_t );

									if ( bboxIntersect( bboxminA, bboxmaxA, bboxminB, bboxmaxB ) )
									{
										deletion_flag = true;
										break;
									}
								}
							
							}
							if(deletion_flag)
								break;
						}	//for( LONG ii=0; ii < m_strokes.size(); ii++)	
					}	//if ( clonethis.GetType() == L"polymsh" )
					
					m_strokes[ l_strokeInd ].flag.push_back( deletion_flag );
				} //end intersection test
				
				LONG j = 0;
				while (j < m_strokes[ l_strokeInd ].transform.size() )
				{
					if  ( m_strokes[ l_strokeInd ].flag[ j ] )
					{
						s_stroke *my_stroke = &m_strokes[ l_strokeInd ];
						my_stroke->flag.erase( my_stroke->flag.begin()+j );
						my_stroke->transform.erase( my_stroke->transform.begin() + j );
						my_stroke->rotx.erase( my_stroke->rotx.begin() + j );
						my_stroke->roty.erase( my_stroke->roty.begin() + j );
						my_stroke->rotz.erase( my_stroke->rotz.begin() + j );
						my_stroke->rotrand.erase( my_stroke->rotrand.begin() + j );
						my_stroke->transrand.erase( my_stroke->transrand.begin() + j );
						my_stroke->sclrand.erase( my_stroke->sclrand.begin() + j );
						my_stroke->posOffset.erase( my_stroke->posOffset.begin() + j );
						my_stroke->dupCount--;
						l_dupcount--;
					}
					else
						j++;
				}
				
				// duplication removed all objects in stroke
				if ( !m_strokes[ l_strokeInd].transform.size() )	{
					m_cloneId = prev_cloneind;
					m_strokes.pop_back();
					return CStatus::False;
				}
			}
			
			calcScaleRampAndFollowStroke( in_ctxt );
	
		}
#ifdef _DEBUG
		logmsg(L"Clone overlaps: " + CString (clock() - timer) );
#endif	
		timer = clock();

		CRefArray l_dupclones;
		if ( !l_dupcount )
		{
			m_cloneId = prev_cloneind;
			m_strokes.pop_back();
			return CStatus::False;
		}
		m_strokes[ l_strokeInd ].dupCount = l_dupcount;
		m_strokes[ l_strokeInd ].strokeSourceObj = m_paintClones [ m_cloneId ];

		if ( l_duplicate )
		{
			// DUPLICATION function	*****************************************
			if ( l_dupcount )
			{
				Duplicate( m_paintClones [ m_cloneId ], l_dupcount, paintinstances, l_dupclones );	
				if( !l_dupclones.GetCount() )	{
					m_strokes.pop_back();
					if ( m_toolMode == kDrawingDrop )
						return CStatus::Fail;
					else
						return CStatus::OK;
				}
				
				m_strokes[ l_strokeInd ].strokeclones = l_dupclones;
			}
			else	// no clones to duplicate
			{
				if ( m_options.listpick == 1 )				
					m_cloneId = prev_cloneind;			
				return CStatus::OK;
			}
			timer = clock();
		
			// UPDATE KINE STATE
			for(LONG i=0; i< l_dupclones.GetCount(); i++)	{
				// Update Kine state
				X3DObject xobj( l_dupclones[i] );
				KinematicState kstateobj( xobj.GetKinematics().GetGlobal() );
			
				if ( ( (  m_options.scaleramp != 0 || m_options.FollowStroke ) &&  m_toolMode != kDrawingDrop ) ||  m_toolMode == kDrawingDrop || m_lineDrawing )
				{
					in_ctxt.BeginKinematicStateUpdate( kstateobj );
#ifdef _DEBUG
								cnta++;
#endif
					in_ctxt.UpdateKinematicState( kstateobj, m_strokes[ l_strokeInd ].transform[ i ] );
				}
				else	{
					kstateobj.PutTransform( m_strokes[ l_strokeInd ].transform[ i ] );
				}
				// Add clone to group
				if ( m_options.groupName != L"" && m_cloneGroup.IsValid() )	
					m_cloneGroup.AddMember( xobj );
			
			}
			// update kine state on rest of strokes
			if ( m_toolMode != kDrawingDrop )
				if ( (  m_options.scaleramp != 0 || m_options.FollowStroke ) && !m_lineDrawing )
					for (LONG i=0; i< m_strokes.size() -1; i++)
						for(LONG j=0; j< m_strokes[i].strokeclones.GetCount(); j++)	{		
							X3DObject xobj( m_strokes[i].strokeclones[j] );
							KinematicState kstateobj( xobj.GetKinematics().GetGlobal() );
							in_ctxt.UpdateKinematicState( kstateobj, m_strokes[ i ].transform[ j ] );
						}
			// Add clones to Null
			if ( m_options.nullName != L"" && m_cloneNullParent.IsValid() )	
					m_cloneNullParent.AddChild( l_dupclones );
			else
				Application().GetActiveSceneRoot().AddChild( l_dupclones );
		}

		return CStatus::OK;
	}
	//******************************************************************************************************************** Mouse Up
	CStatus MouseUp( ToolContext& in_ctxt )
	{
		// Called when mouse button is released
		LONG x, y;
		in_ctxt.GetMousePosition( x, y );
		if ( in_ctxt.IsLeftButtonDown() || in_ctxt.IsRightButtonDown() )
		{
			LONG l_ind = 0;
			if (m_toolMode == kDrawing || m_toolMode == kDrawingDrop)
			{
				clock_t timer = clock();
				if ( ( (  m_options.scaleramp != 0 || m_options.FollowStroke ) && m_toolMode != kDrawingDrop ) || m_toolMode == kDrawingDrop || m_lineDrawing )
					for(LONG j=0;j < m_strokes.size(); j++)
					{
#ifdef _DEBUG			
						s_stroke *cs = &m_strokes[j];
						LONG cnt = cs->strokeclones.GetCount();
						assert( cnt == cs->transform.size() );
						assert( cnt == cs->dupCount );
						assert( cnt == cs->rotrand.size() );
						assert( cnt == cs->rotz.size());				
#endif	
						for(LONG i=0;i<m_strokes[j].strokeclones.GetCount();i++)	{
							X3DObject l_obj = m_strokes[j].strokeclones[i];							
							in_ctxt.EndKinematicStateUpdate( l_obj.GetKinematics().GetGlobal() );
						}
					}
#ifdef _DEBUG
				logmsg(L"End kine state UPDATE time: ", clock() - timer );
#endif
				in_ctxt.SetCursor(siCrossCursor);
			}
			else
				if (m_toolMode == kBrushRadius)	{
					SetCursorPos(lp_mouse.x,lp_mouse.y);
					ShowCursor(TRUE);
					m_brush.SetBaseRadius( m_brush.GetRadius() );
				}

		}
#ifdef _DEBUG
		logmsg( L"Count a: " + CString( cnta ) + L"   count b: " + CString( cntb ) );
#endif
		m_toolMode = kIdle;
		m_activeAxis = kNoDragAxis;
		in_ctxt.Redraw( false );

		return CStatus::OK;
	}

	// ************************************************************************************************************************************************ DRAW Tool Info
	CStatus DrawToolInfo(ToolContext& in_ctxt)
	{
	/*		LONG viewWidth, viewHeight;

			if ( in_ctxt.GetViewSize( viewWidth, viewHeight ) != CStatus::OK )
				return CStatus::Fail;
			in_ctxt.BeginViewDraw();
			glColor3f( 1.0,1.0, 1.0 );
			LONG width = 0, height = 0, descent = 0;
			in_ctxt.GetTextSize( m_piemenu.GetSelectionName(), width, height, descent );
			GLint x = (GLint)m_mouseX - width/2, y = (GLint)m_mouseY - 27;
			glRasterPos2i(x ,y);
			
			in_ctxt.DrawTextString(m_piemenu.GetSelectionName());

			in_ctxt.EndViewDraw();*/
			return CStatus::OK;
	}

	// ************************************************************************************************************************************************ DRAW Highlight

	CStatus DrawComponentHighlight(ToolContext& in_ctxt)
	{
		Property oGApprox;
		if(!m_picked.IsValid())
			return CStatus::Fail;
		//get subdiv level and subdivision method
		m_picked.GetPropertyFromName("geomapprox", oGApprox);
		int approxLevel = oGApprox.GetParameter("gapproxmosl").GetValue();
		int subdrule = m_picked.GetActivePrimitive().GetParameterValue("subdrule");
		
		CMatrix4 l_pickedMatrix(m_picked.GetKinematics().GetGlobal().GetTransform().GetMatrix4() );
		
		glColor4ub( highlightColorR, highlightColorG, highlightColorB,highlightColorA );			
		
		if (m_pickedSubcompType==kEdge) //draw edge
		{
			CVector3Array ve;
			if (approxLevel && (subdrule != 3) ) // approxlevel not 0 and subd rule not linear
				ve = subdivideEdge(m_pickedEdge, (int)1);
			else 
				ve =  m_pickedEdge.GetVertices().GetPositionArray ();

			glLineWidth(2.0f);
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

			glEnable(GL_LINE_SMOOTH);
			glBegin( GL_LINE_STRIP );
				for (LONG i=0;i< ve.GetCount();i++)
				{
					ve[i]*=l_pickedMatrix;
					glVertex3d( ve[i].GetX(),ve[i].GetY(),ve[i].GetZ() );
				}
			glEnd();
		
			glDisable(GL_LINE_SMOOTH);	
			glDisable(GL_BLEND);
		}
		else 
		if (m_pickedSubcompType==kPoly) //poly
		{								
			glEnable( GL_BLEND );
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glEnable ( GL_POLYGON_STIPPLE );
			glPolygonStipple(halftonePolyMask);
			
			if (approxLevel && (subdrule != 3) )		//subdivided poly drawing
			{
				CVector3Array ppos = subdivideFace(m_pickedPolyFace,1);
				CVector3 pos;
				glBegin(GL_TRIANGLE_FAN);
					for(int i=0;i<ppos.GetCount();i++)
					{
						pos = ppos[i];
						pos*=l_pickedMatrix;
						glVertex3d( pos.GetX(),pos.GetY(),pos.GetZ() );
					}
					pos = ppos[1];
					pos*=l_pickedMatrix;
					glVertex3d( pos.GetX(),pos.GetY(),pos.GetZ() );
				glEnd();
			}
			else
				{
					LONG nbTriangles = m_pickedPolyFace.GetNbPoints()-2;
					CLongArray polygonNeiVtxIndices = m_pickedPolyFace.GetVertices().GetIndexArray();
					CLongArray polygonTriSubIndices = m_pickedPolyFace.GetTriangleSubIndexArray();
					PolygonMesh pmesh( m_picked.GetActivePrimitive().GetGeometry() );				
					CVertexRefArray  verts( pmesh.GetVertices() );
									
					CVector3 pos;
					glBegin(GL_TRIANGLES);
						LONG j;
						for(j = 0; j < nbTriangles; j++) 
							for(int k=0;k<3;k++) 
							{
								LONG index = polygonNeiVtxIndices[polygonTriSubIndices[j*3+k]];
								Vertex v(verts[index]);
								CVector3 pos(v.GetPosition()); 
													
								pos*=l_pickedMatrix;	
								glVertex3d( pos.GetX(),pos.GetY(),pos.GetZ() );
							}
					glEnd();
				}
							
			glDisable ( GL_POLYGON_STIPPLE );
			glDisable ( GL_BLEND );					
		}
		else	
			if (m_pickedSubcompType==kVertex) 
			{
				CVector3 pos;
				if (approxLevel && (subdrule != 3) )
					pos = subdividePoint(m_pickedVertex,(int) 1);
				else
					pos = m_pickedVertex.GetPosition();
				pos*=l_pickedMatrix;
				glPointSize(8.0f);
				glEnable(GL_POINT_SMOOTH);
				glBegin( GL_POINTS );
						glVertex3d( pos.GetX(),pos.GetY(),pos.GetZ() );
				glEnd();
				glDisable(GL_POINT_SMOOTH);
			}
		return CStatus::OK;
	}
	CStatus DrawInfoAtCursor(ToolContext& in_ctxt, CString l_str)
	{
			LONG viewWidth, viewHeight;

			if ( in_ctxt.GetViewSize( viewWidth, viewHeight ) != CStatus::OK )
				return CStatus::Fail;
			in_ctxt.BeginViewDraw();
			glColor3f( 1.0,1.0, 1.0 );
			LONG width = 0, height = 0, descent = 0;
			in_ctxt.GetTextSize( l_str, width, height, descent );
			GLint x = (GLint)m_mouseX - width/2, y = (GLint)m_mouseY - 27;
			glRasterPos2i(x ,y);
			
			in_ctxt.DrawTextString( l_str );

			in_ctxt.EndViewDraw();
			return CStatus::OK;
	}
	CStatus DrawInfoAtPosition(ToolContext& in_ctxt, GLint inx, GLint iny, CString l_str)
	{
			LONG viewWidth, viewHeight;

			if ( in_ctxt.GetViewSize( viewWidth, viewHeight ) != CStatus::OK )
				return CStatus::Fail;
			in_ctxt.BeginViewDraw();
			glColor3f( 1.0,1.0, 1.0 );
			LONG width = 0, height = 0, descent = 0;
			in_ctxt.GetTextSize( l_str, width, height, descent );
			GLint x = inx- width/2, y = iny - 27;
			glRasterPos2i(x ,y);
			
			in_ctxt.DrawTextString( l_str );

			in_ctxt.EndViewDraw();
			return CStatus::OK;
	}
	// ************************************************************************************************************************************************ DRAW
	CStatus DrawDebugBrushBBox(ToolContext& in_ctxt,CVector3 center, double l_scale, CLine x, CLine y, CLine z, bool drawinviewplane=true)
	{		
		CVector3 dir;
		if (drawinviewplane)
		{	
			CPlane vp;
			in_ctxt.GetViewPlane(vp);
			dir = vp.GetTangentV();
			dir.NormalizeInPlace();			
			x.SetTangent(vp.GetTangentU());
		}
		else	{	
			dir = z.GetTangent();
			dir.NormalizeInPlace();
		}	
		x.SetOrigin(CVector3(0,0,0));

		CLine pt1(x.GetPosition(-l_scale), dir );
		CLine pt2(x.GetPosition(l_scale), dir );

		CVector3 points[4];
		
		points[0] = pt2.GetPosition(l_scale);
		points[1] = pt2.GetPosition(-l_scale);
		points[2] = pt1.GetPosition(-l_scale);
		points[3] = pt1.GetPosition(l_scale);
		//in_ctxt.BeginViewDraw();
		glPointSize(10.0);
		glBegin(GL_LINE_LOOP);
			for(int i=0;i<4;i++) {
				points[i]+=center;
				glVertex3d(points[i].GetX(),points[i].GetY(),points[i].GetZ() );
			}
		glEnd();
		//in_ctxt.EndViewDraw();
		return CStatus::OK;
	}
	CStatus Draw( ToolContext& in_ctxt )
	{	
		if ( in_ctxt.IsActiveView())
		{
			
			if ( !in_ctxt.IsNavigating() )
			{
				CVector3 l_pos = m_pickedSubcompPos;
				CVector3 x,y,z;
				CLine axis[3];
				CVector3 normal = m_pickedNormal;
				bool gridpicking = false;
				if ( (LONG) ppg.GetParameter(L"highlightpolysel").GetValue() )
					if (m_pickedSubcompType != kNone && !in_ctxt.IsInteracting() )
						DrawComponentHighlight(in_ctxt);
				
				/*if ( m_pickedSubcompType != kNone )
				{			
					AlignPrimitiveToComponentRefFrame(in_ctxt,m_pickedPolyFace,m_picked,x,y,z,false);
					
					axis[0].Set(l_pos,x);
					axis[1].Set(l_pos,y);
					axis[2].Set(l_pos,z);
				}
				*/
				//draw axes

				/*
			
				double l_dScale = GetViewScaleFactor( in_ctxt,l_pos, 1 );	//scale factor for elements that needs to have constant scale in view
					
				glEnable (GL_BLEND);
				glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
				glEnable ( GL_CULL_FACE );
				glDisable ( GL_DEPTH_TEST );
				glEnable (GL_LINE_SMOOTH);
				glDepthMask( GL_FALSE );

				glLineWidth(1.0f);
					
				double axis_len = m_axisScale * l_dScale; //axis drawing length / pix/ 
				const double cone_len = m_axisScale*0.3 * l_dScale; //cone scale
					
				GLdouble mat[16];
				LONG i = 1;
				{
					int colors[4]={0,0,0,0};
					if ( m_options.alignaxis == i ) 	
						colors[3] = 255;					
					else 						
						colors[3] = 50;
						
					CVector3 v  = axis[i].GetPosition(axis_len);
					CVector3 up = axis[i].GetTangent();
					//-----------------------draw lines
						
					colors[i] = 255;
						
					glColor4ub(colors[0],colors[1],colors[2],colors[3]);
					glBegin(GL_LINES);
						glVertex3d(l_pos[0],l_pos[1],l_pos[2]);
						glVertex3d(v[0],v[1],v[2]);
					glEnd();	

					glPushMatrix();
					//-----------------------draw end cones
					CTransformation tr;
					CVector3 r_obj= m_picked.GetLocalScaling();
					CRotation rot;
						
					switch (i){
						case 0: rot.SetFromXYZAxes( axis[2].GetTangent(),axis[0].GetTangent(),axis[1].GetTangent() );break;
						case 1:	rot.SetFromXYZAxes( axis[0].GetTangent(),axis[1].GetTangent(),axis[2].GetTangent() );break;
						case 2:	rot.SetFromXYZAxes( axis[1].GetTangent(),axis[2].GetTangent(),axis[0].GetTangent() );break;
					}
					tr.SetTranslation(v);					
					tr.SetRotation(rot);							
					tr.SetScalingFromValues(cone_len, cone_len*2, cone_len);
					tr.GetMatrix4().Get(mat[0],  mat[1],  mat[2],  mat[3],	mat[4],  mat[5],  mat[6],  mat[7],	mat[8],  mat[9],  mat[10], mat[11],	mat[12], mat[13], mat[14], mat[15]	);
					glMultMatrixd(mat);
							
					ogl_drawCone();
							
					glPopMatrix();
				}
				glDisable (GL_LINE_SMOOTH);
				glDisable ( GL_CULL_FACE );
				glEnable ( GL_DEPTH_TEST );
				glDepthMask( GL_TRUE );
					
				glDisable( GL_BLEND);
				*/

				if ( ((LONG)ppg.GetParameter(L"Paint_Mode").GetValue() == (LONG)0 &&  m_toolMode != kDrawingDrop) || ( m_toolMode == kDrawing ) )	{
					bool screenspacebrush = true;
					CRotation rot;
					double rotangle=0;
					CVector3 rotaxis;
					if(!m_screenspacebrush) {
						rot.SetFromXYZAxes(axis[2].GetTangent(),axis[0].GetTangent(),axis[1].GetTangent());			
						rotaxis = rot.GetAxisAngle(rotangle);
					}
					else	{
						CPlane cp;
						in_ctxt.GetViewPlane(cp);
						rot.SetFromXYZAxes(cp.GetTangentU(),cp.GetTangentV(),cp.GetNormal());		
						rotaxis = rot.GetAxisAngle(rotangle);
					}
					double scale = GetViewScaleFactor(in_ctxt, m_pickedSubcompPos, m_brush.GetRadius() );
					scale = m_brush.GetRadius();
					m_brush.Draw(in_ctxt,scale,m_pickedSubcompPos.GetX(),m_pickedSubcompPos.GetY(),m_pickedSubcompPos.GetZ(), rotangle, rotaxis);
					if (m_toolMode == kBrushRadius)
						DrawInfoAtPosition(in_ctxt, m_mouseDownX, m_mouseDownY, L"Radius: "+CString(scale) );
					
				}
				// draw Drop helpers
				if ( m_toolMode == kDrawingDrop ){
					
					CRotation crot;
					crot.SetFromXYZAxes( m_pickedFrameAxisX,m_pickedFrameAxisY,m_pickedFrameAxisZ);
					double angle = 0;
					CVector3 axis = crot.GetAxisAngle(angle);
					angle = RadiansToDegrees(angle);
					
					CPlane cp;
					cp.Set(m_pickedPosition, m_pickedFrameAxisY );

					CVector3 mousedragvect, md ;
					in_ctxt.SetManipulationPlane( cp );
					in_ctxt.GetWorldPosition(m_mouseDownX,m_mouseDownY, md);
					in_ctxt.GetWorldPosition(m_mouseDragX,m_mouseDragY, mousedragvect);
					in_ctxt.ClearManipulationPlane();
					CVector3 len = mousedragvect ;
					len -= md;
					
					double viewScale = GetViewScaleFactor( in_ctxt, md, 1 );
					double radius = abs ( len.GetLength() );
					glEnable( GL_BLEND );
					glDisable (GL_DEPTH_TEST );
					glBlendFunc ( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
					glLineWidth( 1.0f );
					glColor4d(.8,.8,.9,.5);
					glPushMatrix();
					glTranslated(md[0],md[1],md[2]);
					
					glRotated(angle, axis.GetX(),axis.GetY(),axis.GetZ() );
						ogl_drawCircle( viewScale,20, true);	// small circle
						ogl_drawCircle( radius ,50, true);
						glBegin( GL_LINES );
							glVertex3d( 0, 0, 0 );
							glVertex3d( radius, 0, 0 );
						glEnd();
					glPopMatrix();
					glColor4d(.8,.7,.9,.7);
					glBegin( GL_LINES );
						glVertex3d( md[0], md[1], md[2] );
						glVertex3d( mousedragvect[0], mousedragvect[1], mousedragvect[2] );
					glEnd();
					glEnable (GL_DEPTH_TEST );
					glDisable( GL_BLEND );
					
				}
				//info
				DrawToolInfo(in_ctxt);
			}
	
			// draw buttons
			if ( !in_ctxt.IsInteracting() || m_toolMode == kButtonDrag )
			{			
				/*
				m_alignButtons.Draw(in_ctxt);
				m_axisButtons.Draw(in_ctxt);
				m_toolscopeButtons.Draw(in_ctxt);
				m_additionalButtons.Draw(in_ctxt);
				*/
			}
		}
		LogOpenGLErrors();
		return CStatus::OK;
	}

	CStatus GetGridData(ToolContext& in_ctxt, LONG x,LONG y, CVector3 &out_pos)	
	{
		CPlane gridPlane;
				
		in_ctxt.GetGridPlane(gridPlane);	
		
		in_ctxt.SetManipulationPlane(gridPlane);

		CStatus gridPickValid = in_ctxt.GetWorldPosition(x,y,out_pos);
		in_ctxt.ClearManipulationPlane();
		return gridPickValid;
	}

	CStatus GetGridData(ToolContext& in_ctxt, LONG x,LONG y, CVector3 &out_pos, CPlane &out_plane)	
	{
		in_ctxt.GetGridPlane(out_plane);	
		
		in_ctxt.SetManipulationPlane(out_plane);

		CStatus gridPickValid = in_ctxt.GetWorldPosition(x,y,out_pos);
		in_ctxt.ClearManipulationPlane();
		return gridPickValid;
	}
	//**************************************************************************************************************** Mouse MOVE
	CStatus MouseMove( ToolContext& in_ctxt )
	{
		// Called when mouse is moved without any buttons pressed
		CStatus l_status = CStatus::False;
		in_ctxt.GetMousePosition( m_mouseX, m_mouseY);
		
		clock_t now=clock() - m_tooltime;
		
		if (now > MOUSE_MOVE_TIME_DELAY) {
			clock_t timer = clock();
			if ( (LONG) ppg.GetParameter(L"highlightpolysel").GetValue() )	
				l_status = GetPickData(in_ctxt,m_mouseX,m_mouseY,m_pickedSubcompPos);
			
				// no object under cursor, check for grid
				if (l_status != CStatus::OK)
					GetGridData(in_ctxt, m_mouseX,m_mouseY,m_pickedSubcompPos) == CStatus::OK ? true : false;
			
			in_ctxt.Redraw( false );
			m_tooltime=clock();
			m_lastMouseMoveStatus = l_status;
		};
		
		return CStatus::OK;
	}

	CStatus SnapValid( ToolContext& in_ctxt )
	{
		
		X3DObject xobj = in_ctxt.GetSnapObject();
		for (LONG i=0;i< m_strokes.size(); i++)
			for (LONG j=0; j < m_strokes[i].strokeclones.GetCount(); j++)
			{
				X3DObject sobj( m_strokes[i].strokeclones[j] );

				if(xobj.GetFullName() == sobj.GetFullName() )
					return CStatus::Abort;
			}		
		return CStatus::OK;
	}
	bool IsModKeyUp(ToolContext &in_ctxt)
	{
		return in_ctxt.IsRightButtonDown();
	}

	

	// ******************************************************************************************************** CLEAR DATA

	void ClearPickedData()
	{		
		m_pickedSubcompType = kNone;	
		m_picked.ResetObject();
		m_pickedEdge = Edge();
		m_pickedPolyFace = PolygonFace();
		m_pickedVertex = Vertex();
		m_activeAxis = kNoDragAxis;
	}
	void ClearData()
	{
#ifdef _DEBUG
		logmsg(L"clear data called ");
#endif
		ClearPickedData();
		m_strokes.clear();
		m_paintClones.Clear();	
	}
	void ClearAllData()
	{	
		ClearPickedData();
		ClearData();	
	}
	
	// ******************************************************************************************************************************************* GET SNAP DATA
	X3DObject GetPolyComponent(ToolContext& in_ctxt,CRef &target, PolygonFace &out_polyface)
	{			
			X3DObject l_meshObject;
			CComAPIHandler cCollItem(target);
			CComAPIHandler cSubComp(cCollItem.GetProperty(L"SubComponent"));
			CComAPIHandler cCompColl(cSubComp.GetProperty(L"ComponentCollection"));
			CValueArray indices = cSubComp.GetProperty( L"ElementArray" );
			l_meshObject = cSubComp.GetProperty( L"Parent3DObject" ); 
			
			PolygonMesh pmesh( l_meshObject.GetActivePrimitive().GetGeometry() );
			CString l_subcomp = cSubComp.GetProperty(L"Type");
			if(LONG(indices.GetCount() ) == 1L)
				if ( l_subcomp == L"polySubComponent" )	{
					out_polyface = pmesh.GetPolygons().GetItem((LONG)indices[0]);
					m_pickedSubcompType = kPoly;
				}
			return l_meshObject;
	}

	CStatus GetGeometrySubcomponentData(ToolContext& in_ctxt, LONG x, LONG y, CRefArray& in_objects, PolygonFace& in_poly, CVector3& out_pickedPos, X3DObject& out_picked, CVector3& out_pickedNormal)
	{
		CLongArray l_points;
		l_points.Add(x);
		l_points.Add(y);
		CRefArray components;
		X3DObject l_picked = in_objects[0];
		in_ctxt.Pick( l_points, siPickSingleSubComponent, siPickRaycast, siPolygonFilter, in_objects, components );
		if( components.GetCount() != 0 )
		{
			out_picked	 =  GetPolyComponent(in_ctxt,components[0], in_poly);
			in_ctxt.GetWorldPosition(x,y,out_pickedPos);
			out_pickedNormal = getPolyNormal(in_ctxt,in_poly, false);				
		}	//end if components picked
		else
			return CStatus::False;
		return CStatus::OK;
	}
	CStatus GetPickData(ToolContext& in_ctxt, LONG x, LONG y, CVector3 &l_primPos)
	{
		ClearPickedData();

		CRefArray	selection = Application().GetSelection().GetArray();
		CStringArray l_snapFamily;

		CLongArray l_points;
		l_points.Add( x );
		l_points.Add( y );
		CRefArray objects, in_objects;

		if ( (LONG)ppg.GetParameter(L"paint_on_selection_only").GetValue() != 0 && selection.GetCount() )				 
			in_objects = ExpandSelection(in_ctxt, selection);
		
		PickBuffer pb = in_ctxt.GetPickBuffer(x,y,3,3,siObjectFilter, in_objects);
		CRef pbobj = pb.GetObjectAtPosition(x,y);
		objects.Add(pbobj);
	//	in_ctxt.Pick( l_points, siPickSingleObject, siPickRaycast, siPolyMeshFilter, in_objects, objects );
		
		CStatus pickStatus = CStatus::False;
		
		if ( objects.GetCount() > 0 ) 
		{ 	
			CString objclass = objects[0].GetClassIDName();
			if (objclass== L"X3DObject")
			{
				X3DObject l_picked = objects[0];
				CString l_pickedType = l_picked.GetType();
				
				if (l_pickedType == L"polymsh")		{	
					pickStatus = GetGeometrySubcomponentData(in_ctxt,x,y,objects,m_pickedPolyFace,l_primPos, m_picked, m_pickedNormal);
					if( pickStatus != CStatus::OK	)
						return pickStatus;
				}
			}		//if x3dobject
				
		}	// end	 if ( objects.GetCount() > 0 ) 
		else	
			if (m_options.paintgrid)	{
				
				CPlane gridPlane;
				in_ctxt.GetGridPlane(gridPlane);	

				in_ctxt.SetManipulationPlane(gridPlane);
				
				pickStatus = in_ctxt.GetWorldPosition(x,y,l_primPos) ;
				in_ctxt.ClearManipulationPlane();
				
				// Reject intersection points behind the camera
				if (pickStatus == CStatus::OK)	{
					CVector3 cameraPoint;
					if ( in_ctxt.WorldToCamera( l_primPos, cameraPoint ) != CStatus::OK || cameraPoint.GetZ() > 0.0 )
						pickStatus = CStatus::Fail;
				}
				
		}
		return pickStatus;
	}
	
	CStatus KeyUp( ToolContext& in_ctxt )
	{	
		/*
		switch ( (ULONG) in_ctxt.GetShortcutKey() ) 		{
			
			default:
			break;
		}*/
		return CStatus::OK;
	}
	CStatus KeyDown( ToolContext& in_ctxt )
	{

		return CStatus::OK;
	}

}; // end class

////////////////////////////////////////////////////////////////////////////////
//
// Plugin Callbacks
//
////////////////////////////////////////////////////////////////////////////////
SICALLBACK pmMeshPaint_Menu_Init( CRef& in_ctxt )
{
	Context ctxt( in_ctxt );
	MenuItem menuItem;
	Menu menu = ctxt.GetSource();
	menu.AddCommandItem( L"Mesh Paint", L"pmMeshPaint", menuItem );
	return true;
}

SICALLBACK pmMeshPaint_Init( CRef& in_ctxt )
{
	pmMeshPaint *l_pTool = new pmMeshPaint;
	if ( !l_pTool ) return CStatus::Fail;
	ToolContext l_ctxt( in_ctxt );
	l_ctxt.PutUserData( CValue((CValue::siPtrType)l_pTool) );
	return CStatus::OK;
}

SICALLBACK pmMeshPaint_Term( CRef& in_ctxt )
{
	ToolContext l_ctxt( in_ctxt );
	pmMeshPaint *l_pTool = (pmMeshPaint *)(CValue::siPtrType)l_ctxt.GetUserData();
	if ( !l_pTool ) return CStatus::Fail;
	delete l_pTool;
	l_ctxt.PutUserData( CValue((CValue::siPtrType)NULL) ); // Clear user data
	return CStatus::OK;
}


// Use a macro to simplify callback setup and forwarding...

#define DECLARE_TOOL_CALLBACK(TOOL,CALLBACK) \
SICALLBACK TOOL##_##CALLBACK( ToolContext& in_ctxt ) { \
	TOOL *l_pTool = (TOOL *)(CValue::siPtrType)in_ctxt.GetUserData(); \
	return ( l_pTool ? l_pTool->CALLBACK( in_ctxt ) : CStatus::Fail ); }

DECLARE_TOOL_CALLBACK( pmMeshPaint, Setup );
DECLARE_TOOL_CALLBACK( pmMeshPaint, Cleanup );
DECLARE_TOOL_CALLBACK( pmMeshPaint, Activate );
DECLARE_TOOL_CALLBACK( pmMeshPaint, Deactivate );
DECLARE_TOOL_CALLBACK( pmMeshPaint, MouseDown );
DECLARE_TOOL_CALLBACK( pmMeshPaint, MouseDrag );
DECLARE_TOOL_CALLBACK( pmMeshPaint, MouseUp );
DECLARE_TOOL_CALLBACK( pmMeshPaint, Draw );
DECLARE_TOOL_CALLBACK( pmMeshPaint, MouseMove );
DECLARE_TOOL_CALLBACK( pmMeshPaint, SnapValid );
DECLARE_TOOL_CALLBACK( pmMeshPaint, KeyUp );
DECLARE_TOOL_CALLBACK( pmMeshPaint, KeyDown );

